#
# This script displays a map of the selected product. 
# All timesteps of a file are plotted in separate png-files.  
# Prepare the your netcdf-files with the R-script "Prep.Data.R" or "Apply.Function.R"
###########################################################################################

# Name of the file
filename = "YOURFILENAME.nc"

# Select from the Product
product <- "SIS"

# If you want to create a png-file, you need to specify the filename (without .png)
picout <- FALSE
pic <- "YOURPICFILENAME"

# The data directory needs to be adjusted
datadir <- "YOURDATADIRECTORY"
 #datadir <- "C:/Documents/jtrentma/CMSAF_V2/Data/"

# The CMSAF-R-TOOLBOX directory needs to be adjusted
# This is only needed if picout <- TRUE
toolboxdir <- "TOOLBOXDIRECTORY"
 #toolboxdir <- "C:/Documents/jtrentma/CMSAF-R-TOOLBOX/

# In case you want to add something to the title of the plot
add2title <- "CM SAF, "

# select the plot region
lonmin <- 0
lonmax <- 10
latmin <- 5
latmax <- 15

# Set the size of the output window (in # of pixels)
height <- 600
width <- 600

# Select to plot an anomaly (use of appropriate color scale)
plot.ano <- FALSE

# Set the breaks of the color bar manually
# Suggested for plotting of anomalies
set.col.breaks <- FALSE

  #brk.set <- c(-50,-20,-10,-5,-2.5,2.5,5,10,20,50)
  brk.set <- seq(240,310,5)

  # select the range of the color bar
  # select -99 for flexible range
  colmin0 <- -99
  colmax0 <- -99

  # Set the number of colors
  ncol <- 11

# Set to true if higher resolution additional data is required
plotHighRes <- FALSE

# Switches to determine additional plot layers
plotCoastline <- TRUE
plotCountries <- TRUE
plotRivers <- FALSE
  # Thickness of the additional lines
  contour.thick <- 2.

# Switches to control the display of cities
plotCities <- FALSE
  # Symbol
  pch.cities <- 2
  
  # Size of symbol
  cex.cities <- 1.
  
  # Name next to the sysmbol?
  label.cities <- TRUE
  
  # Capital only
  plotCapitals <- 1
   # 0 = no capitals,1 = country capitals, 2 = provincial, state, regional capitals, 
   # 3 = local capitals
  
  # Size of the label
  cex.label.cities <- 0.5
  
  # Lat offset of label
  dlat <- 0.25

# For HLW and HSH select the variable and the vertical level
# if not done before with levbox_mergetime
# HLW: level: 1-5, variable: LPWm, RHWm, Tm
# HSH: level: 1-6, variable: Qm, Tm
variable <- "Tm"
level <- 5

# For Cloud Type (CTY) specify the type that should be plotted!
# 1 = Low Clouds, 2 = Middle level clouds, 3 = High opaque clouds
# 4 = High Semitransparent clouds, 5 = Fractional Clouds
CTY.type <- 4

#-------------------------------------------------------------------
# END OF USER DEFINE SECTION. NO CHANGES REQUIRED BEYOND THIS POINT
#-------------------------------------------------------------------






file <- file.path(datadir,filename)

# Make sure that you have all neccessary packages installed! 
suppressMessages(library(cmsaf))
suppressMessages(library(maptools))
suppressMessages(library(RColorBrewer))
suppressMessages(library(colorRamps))
suppressMessages(require(mapdata))
suppressMessages(library(rworldmap))
suppressMessages(library(rworldxtra))

# Set the plot ranges
lonplot=c(lonmin,lonmax)
latplot=c(latmin,latmax)

# Define the HOAPS2011 data set
HOAPS2011 <- c("PRE","EMP","EVA","LHF","NSH","SWS")

# Open the netcdf-file
nc <- nc_open(file)

# Retrieve the name of the variable and the data
datalev <- 1
if (product %in% HOAPS2011) datalev <- 2
if (product == "CTY") datalev <- CTY.type

# Set the variable name
varname <- nc$var[[datalev]]$name

# In HLW and HSH multiple variables are stored in each file
if (product == "HLW" || product =="HSH") {
	varname <- paste(product,"_",variable,sep="")
	}

# Retrieve the unit, the missing_data-value, and the title of the data
unit <- ncatt_get(nc, varname,"units")$value
missval <- ncatt_get(nc,varname,"_FillValue")$value

if (ncatt_get(nc,varname,"title")$hasatt==TRUE) {
	name <- ncatt_get(nc,varname,"title")$value 
	} else {
	 name <- varname
	}
	
# The offset and the scalefactor is required because
# the Fill_Value attribute is not applied by the ncdf-package
# The offset and scalefactor is automatically applied to all data
offset.value <- 0.
scale.factor <- 1.
if (ncatt_get(nc,varname,"add_offset")$hasatt==TRUE) {
	offset.value <- ncatt_get(nc,varname,"add_offset")$value }
if (ncatt_get(nc,varname,"scale_factor")$hasatt==TRUE) {
	scale.factor <- ncatt_get(nc,varname,"scale_factor")$value }

#--------------------------------------------------#

# Determine the grid
londim <- nc$dim[["lon"]]
lon <- londim$vals
nx <- londim$len
latdim <- nc$dim[["lat"]]
lat <- latdim$vals
ny <- latdim$len
	
#--------------------------------------------------#

# Retrieve the time variable
timedim <- nc$dim[["time"]]
nt <- timedim$len
time.unit <- timedim$units
time <- timedim$vals

# Create a R-date-object 
date.time <- as.Date(get_time(time.unit,time))
lat.inv <- FALSE

for (itime in 1:nt) {
  
# Read in the data 
  field <- ncvar_get(nc, varname,start=c(1,1,itime),count=c(nx,ny,1))	
  
# Set the missing data to NA
# Since the data has been corrected for the scalefactor and the offset, 
# this has also to be considered for the FillValue
  na.ind <- which(field==(missval*scale.factor + offset.value))
  field[na.ind] <- NA

# Format the time step for plotting
if (nt > 1) {
  zdate <- paste(format(date.time[itime],"%Y%m"),sep=" ")
  if (nt == 12) zdate <- paste(format(date.time[itime],"%m"),format(date.time[itime],"%B"),sep=" ")
	} else zdate <- ""
	 
# Set the variable to make the plot
  z <- field

# Invert the latitude dimension if necessary
  if (lat[ny] < lat[1]) {
    sort.out <- sort(lat,index.return=TRUE)
    lat <- sort.out$x
    lat.inv <- TRUE
  }
  if (lat.inv) z <- z[,sort.out$ix]
  
# Calulate the mean, min, max for the selected region only
  lon.reg <- which(lon >= lonmin & lon <= lonmax)
  lat.reg <- which(lat >= latmin & lat <= latmax)
  z.reg <- z[lon.reg,lat.reg]

# Print out the mean value of the field
  print(zdate)
  print(paste("Mean: ",mean(z.reg,na.rm=TRUE),sep=''))
  print(paste("Max: ",max(z.reg,na.rm=TRUE),sep=''))
  print(paste("Min: ",min(z.reg,na.rm=TRUE),sep=''))

# Set the title of the plot
  title <- name

#----------------------------------------------------------
# Open a png-file

  picfile <- paste(pic,zdate,".png",sep="")
  picdir <- file.path(toolboxdir,"figures",picfile)
  if (picout == TRUE) png(file=picdir,height=height,width=width)

  colmin <- colmin0
  colmax <- colmax0

  if ((as.integer(colmin) == -99) && (as.integer(colmax) == -99)) {
		colmin <- min(z.reg,na.rm=TRUE)
		colmax <- max(z.reg,na.rm=TRUE)
		}
  
  if (set.col.breaks) {
		brk <- brk.set
		} else {
		brk <- seq(colmin,colmax,length.out=ncol+1)
		} 
  
# Set the colors and the color bar for the Difference plots
col.breaks <- brk
ncolor <- length(col.breaks)
at.ticks <- seq(1,ncolor)
names.ticks <- round(col.breaks[at.ticks])
zlim <- c(1,ncolor)
	
colors <- matlab.like(ncolor-1)
if (plot.ano) colors[as.integer(ncolor/2)] <- rgb(1,1,1)

# Generate the field to be plotted
field.plot <- matrix(ncol=ny,nrow=nx)
for (l in seq(1,ncolor-1) ) {
  idx <- which(z >= col.breaks[l] &
               z < col.breaks[l+1],arr.ind=TRUE)
  field.plot[idx] <- l + 0.1
}

if (plotCoastline&plotCountries)(plotCoastline=FALSE)

# Make the plot including color bar
image.plot(lon,lat,field.plot,xlab="longitude, deg E",ylab="latitude, deg N",
           main=paste(title," (",unit,"), ",add2title,zdate,sep=""),
           legend.mar = 4,xlim=lonplot,ylim=latplot,zlim=zlim,
           nlevel=ncolor-1,col=colors,lab.breaks=names.ticks)

if (plotHighRes){  

 data(worldMapEnv)
 data(worldHiresMapEnv)
 data(countriesHigh)
 world <- as(countriesHigh,"SpatialLines")

 # add rivers
  if (plotRivers) {
    map('rivers', add=TRUE, col="blue")
  }

  # add coastline
  if (plotCoastline) {
    map('worldHires', add=TRUE, interior=F)
  }

  # add country borders
  if (plotCountries) {
    plot(world,add=TRUE)
  }

  # add cities
  if (plotCities) {    
    if (label.cities){ 
      map.cities(pch=pch.cities,cex=cex.cities,capitals=plotCapitals,label=TRUE)
    }else{map.cities(pch=pch.cities,cex=cex.cities,capitals=plotCapitals,label=FALSE)}
    }
}else{

 data(worldMapEnv)

 # add rivers
  if (plotRivers) {
     map('rivers', add=TRUE, col="blue")
  }

  # add coastline
  if (plotCoastline) {
    map('world', add=TRUE, interior=F)
  }

  # add country borders
  if (plotCountries) {
    data(countriesLow)
    world <- as(countriesLow,"SpatialLines")
    plot(world,add=TRUE)
  }

  # add cities
  if (plotCities) {    
    if (label.cities){ 
      map.cities(capitals=plotCapitals,label=TRUE)
    }else{map.cities(pch=pch.cities,cex=cex.cities,capitals=plotCapitals,label=FALSE)}
    }
 }

# Draw lines around the plot
axis(1,lwd=1,at=c(lonmin,lonmax),tick=TRUE,lwd.ticks=0,labels=FALSE)
axis(2,lwd=1,at=c(latmin,latmax), tick=TRUE,lwd.ticks=0,labels=FALSE)
axis(3,lwd=1,at=c(lonmin,lonmax),tick=TRUE,lwd.ticks=0,labels=FALSE)
axis(4,lwd=1,at=c(latmin,latmax), tick=TRUE,lwd.ticks=0,labels=FALSE)

# close the jpg-file
if (picout == TRUE) dev.off()

}
		
# Close the file
  nc_close(nc)
