# Make a plot of a 1D time series 
# Prepare the your netcdf-files with the R-script "Prep.Data.R"

# Name of data file
filename = "YOURFILENAME.nc"

# Select from the Product
product <- "SIS"

# For HLW and HSH select the variable and the vertical level
# if not done before with levbox_mergetime
# HLW: level: 1-5, variable: LPWm, RHWm, Tm
# HSH: level: 1-6, variable: Qm, Tm
variable <- "LPWm"
level <- 4

# For Cloud Type (CTY) specify the type that should be plotted!
# 1 = Low Clouds, 2 = Middle level clouds, 3 = High opaque clouds
# 4 = High Semitransparent clouds, 5 = Fractional Clouds
CTY.type <- 4

# If you want to create a png-file, you need to specify the filename
picout <- FALSE
picfile <- "YOURPICFILENAME.png"

# The data directory needs to be adjusted
datadir <- "YOURDATADIRECTORY"
#datadir <- "C:/Documents/jtrentma/CMSAF_V2/Data/"

# The CMSAF-R-TOOLBOX directory needs to be adjusted
# This is only needed if picout <- TRUE
toolboxdir <- "TOOLBOXDIRECTORY"
#toolboxdir <- "C:/Documents/jtrentma/CMSAF-R-TOOLBOX/"

#----------------------------------------------------------------
# NO CHANGES REQUIRED BEYOND THIS POINT
#----------------------------------------------------------------

picdir <- file.path(toolboxdir,"figures")
file <- file.path(datadir,filename)

# Make sure you have the ncdf4- and RNetCDF-packages installed on your R-installation!
library(cmsaf)

# Define the HOAPS2011 data set
HOAPS2011 <- c("PRE","EMP","EVA","LHF","NSH","SWS")

# Open the netcdf-file
nc <- nc_open(file)

# Retrieve the name of the variable and the data
datalev <- 1
if (product %in% HOAPS2011) datalev <- 2
if (product == "CTY") datalev <- CTY.type

varname <- nc$var[[datalev]]$name

if (product == "HLW" || product =="HSH") {
	varname <- paste(product,"_",variable,sep="")
	}

# Read in the data
field <- ncvar_get(nc, varname)
if(length(dim(field))==2){
  if (product == "HLW" || product =="HSH") {
	  field <- field[level,]
  }
}
	
unit <- ncatt_get(nc, varname,"units")$value
missval <- ncatt_get(nc,varname,"_FillValue")$value
if (ncatt_get(nc,varname,"title")$hasatt==TRUE) {
	name <- ncatt_get(nc,varname,"title")$value 
	} else {
	 name <- product 
	}
	
# The offset and the scalefactor is required because
# the Fill_Value attribue is not applied by the ncdf-package
# The offset and scalefactor is automatically applied to all data
offset.value <- 0.
scale.factor <- 1.
if (ncatt_get(nc,varname,"add_offset")$hasatt==TRUE) {
	offset.value <- ncatt_get(nc,varname,"add_offset")$value }
if (ncatt_get(nc,varname,"scale_factor")$hasatt==TRUE) {
	scale.factor <- ncatt_get(nc,varname,"scale_factor")$value }
	
# Close the file
nc_close(nc)

# Set the missing data to NA
# Since the data has been corrected for the scalefactor and the offset, 
# this has also to be considered for the FillValue
na.ind <- which(field== (missval*scale.factor + offset.value))
field[na.ind] <- NA
#--------------------------------------------------#

# determine the location 
londim <- nc$dim[["lon"]]
lon <- londim$vals
latdim <- nc$dim[["lat"]]
lat <- latdim$vals
if (product == "HLW" || product =="HSH") {
	levdim <- nc$dim[["lev"]]
	level.out <- levdim$vals[level]
	}
#--------------------------------------------------#

# retrieve the time variable
timedim <- nc$dim[["time"]]
nt <- timedim$len
time.unit <- timedim$units
time <- timedim$vals

# Create a R-date-object 
date.time <- get_time(time.unit,time)

#--------------------------------------------------#

title <- name
if (product == "HLW" || product =="HSH") {
	title <- paste(varname,", level: ",level.out,sep="")
	}

# Open a png-file
if (picout == TRUE) png(file=file.path(picdir,picfile))

# set the number of rows and columns of the plot
par(mfrow = c(1,1))

# Determine the min and max of the plotrange
pmin <- min(field,na.rm=TRUE)
pmax <- max(field,na.rm=TRUE)

#plot the data
plot(date.time,field, ylab=paste(title," (",unit,")",sep=""), xlab="Time",
        main=paste(title,", ",format(lat,digits=4,nsmall=2)," N, ",
          format(lon,digits=4,nsmall=2)," E",sep=""),
        ylim=c(pmin,pmax), type="l")

print(summary(field))

# close the jpg-file
if (picout == TRUE) dev.off()
