#
# This script will help you to apply functions of the 'cmsaf' R-package.
# The outfile(s) will be placed in `CMSAF-R-TOOLBOX/output`.
# Be careful when you enter options and check it before hitting enter.
# For usage of this script cmsaf R-package in version 1.8 or higher 
# is required.
# Apply.Function.R contains only a selection of the most important functions 
# of the 'cmsaf' R-package. Please check also the other funtions 
# (https://cran.r-project.org/web/packages/cmsaf/cmsaf.pdf).
#
# Have fun with the CM SAF R TOOLBOX!         (Steffen Kothe / CM SAF 2018-07-18)
#_________________________________________________________________________________
# Start user define section #

  # Optional path to CM SAF R TOOLBOX (no changes needed)
  # If ptt is not defined by the user, the path of the current script is taken
    ptt <- "NoNeedToChange"
  # e.g., ptt <- "C:/cmsaf/CMSAF-R-TOOLBOX"
  # default setting is: ptt <- "NoNeedToChange"
  
      
  # Optional input for Visualizer (no changes needed)
    textsize    <- 1.2
    linesize    <- 1.5
    bordercolor <- "gray20"
    imagewidth  <- -1         # if -1: image dimensions are taken from data
    imageheight <- -1         # a reasonable number would be 800
    na.color    <- "gray80"
  
  # Option for file output (no changes needed)
    nc34        <- 3          # output file format NetCDF3 or NetCDF4 (3 or 4)           

#_________________________________________________________________________________
# End of user define section - No changes necessary beyond this point #

  
  


# Load libraries

  suppressMessages(library(cmsaf,warn.conflicts=FALSE,quietly=TRUE))
  
# Check ptt
  
if (ptt == "NoNeedToChange") {
  
  slash <- "n"

# Get path of current file
  t <- try(ptt <- dirname(sys.frame(1)$ofile),TRUE)
  if("try-error" %in% class(t)|t==".") (ptt <- getwd())
  if (substr(ptt,1,1)=="/") (slash <- "t")

# Get R-Toolbox path
  split_path <- function(path, mustWork = FALSE, rev = FALSE) {
    output <- c(strsplit(dirname(normalizePath(path,mustWork = mustWork)),
                                             "/|\\\\")[[1]], basename(path))
    ifelse(rev, return(rev(output)), return(output))
  }
  dummy <- split_path(ptt)
  
  checkstring <- "CMSAF-R-TOOLBOX"
  n <- NULL
  
  for (i in 1:length(dummy)){
    if (grepl(checkstring,dummy[i])) n <- i
  }
  
  if (!is.null(n)){
    ptt <- do.call("file.path",c(as.list(dummy[1:n])))
    if (slash=="t") ptt <- file.path("/",ptt)
    
    # Test ptt
    if (!dir.exists(file.path(ptt,"data")) | !dir.exists(file.path(ptt,"figures")) |
        !dir.exists(file.path(ptt,"output"))) {
      stop(cat(paste("Something is wrong with your CMSAF-R-TOOLBOX path: ",ptt,sep="")),"\n")
    }
  }
} # end ptt check

  # Test ptt
  if (nchar(ptt)>260)(warning("Your CMSAF-R-TOOLBOX path might be too long!"))
  
  cat("Your CMSAF-R-TOOLBOX path is: ",ptt,sep="","\n")

# Get infile name

   cat("Please choose a NetCDF file (e.g., 'SIS_monthly_means.nc'", "\n")
     infile_name <- as.character(NULL)
     setwd(file.path(ptt,"output"))
     if(exists("selected_file")){
       if(selected_file!="NoFile"){
         infile_org <- selected_file
       } else {
         infile_org <- file.choose()
       }
     } else {
       infile_org <- file.choose()
     }
     
     infile_name <- basename(infile_org)
     if(identical(infile_name, character(0))){
      stop(cat("No infile!",var,sep="","\n"))
     }

# Get information of infile

  infile <- infile_name
  if (file.exists(file.path(ptt,"data",infile_name)))(ddir <- "data")
  if (file.exists(file.path(ptt,"output",infile_name)))(ddir <- "output")
  if ((!file.exists(file.path(ptt,"data",infile_name)))|
     (!file.exists(file.path(ptt,"output",infile_name)))){
       infile <- infile_org
       ncinfo(infile,"s")
  } else {
     ncinfo(file.path(ptt,ddir,infile_name),"s")    
    }

# Select variable

  id <- nc_open(infile)
    vn <- names(id$var)
  nc_close(id)
  
  var_default <- subset(vn, !(vn %in% c("lat","lon","time_bnds","nb2","time")))
  
  # Stop if data are in sinusoidal projection
  
  if ("sinusoidal" %in% vn){
    sorry <- c("Sorry, the CM SAF R Toolbox can not handle data in sinusoidal projection.",
               "Please use the 'change projection' option during the order process.",
               "Your NetCDF data have to be on a regular lon-lat grid.")
    stop(writeLines(strwrap(sorry,width=80,indent=1)),call.=F)
  }
  
  if (length(var_default)==1){
    var <- var_default
    cat("\n")
    cat("Variable ",var," will be used.",sep="","\n")
    cat("\n")
  } else {
      cat("\n")
      cat("Please enter variable (or push ENTER): ", "\n")
      var <- scan(file="",what=character(),n=1,quiet=TRUE)
      if(identical(var, character(0))){
        var <- var_default[1]
        cat("variable = ",var,sep="","\n")
      } else {
          if (toupper(var) %in% toupper(var_default)){
            var <- var_default[which(toupper(var)==toupper(var_default))]
          } else {
              var <- var_default[1]
              cat("Variable ",var," will be used.",sep="","\n")
              cat("\n")
            }
      }
  }

# Execute optionally functions

go_on <- "y"

repeat{
  
  if (toupper(go_on)!="Y") break

  cat("Please choose an operator:", "\n")

  cat( "\n","cmsaf.mulc___(1)  . . . . . . Multiply data with a constant",sep="", "\n")
  cat("divdpm_______(2)  . . . . . . Divide by days per month", "\n")
  cat("fldmax_______(3)  . . . . . . Determine the field maximum", "\n")
  cat("fldmean______(4)  . . . . . . Determine the field mean", "\n")
  cat("fldmin_______(5)  . . . . . . Determine the field minimum", "\n")
  cat("mon.anomaly__(6)  . . . . . . Determine monthly anomalies", "\n")
  cat("monmean______(7)  . . . . . . Determine monthly means", "\n")
  cat("monsum_______(8)  . . . . . . Determine monthly sums", "\n")
  cat("multimonmean_(9)  . . . . . . Determine multi-monthly means", "\n")
  cat("multimonsum__(10) . . . . . . Determine multi-monthly sums", "\n")
  cat("muldpm_______(11) . . . . . . Multiply by days per month", "\n")
  cat("seas.anomaly_(12) . . . . . . Determine seasonal anomalies", "\n")
  cat("seasmean_____(13) . . . . . . Determine seasonal means", "\n")
  cat("sellonlatbox_(14) . . . . . . Select a region by longitude and latitude", "\n")
  cat("selmon_______(15) . . . . . . Extract a list of months", "\n")
  cat("selperiod____(16) . . . . . . Extract a list of dates", "\n")
  cat("selpoint_____(17) . . . . . . Extract data at a given point", "\n")
  cat("selyear______(18) . . . . . . Extract a list of years", "\n")
  cat("timmean______(19) . . . . . . Determine mean of a timeseries", "\n")
  cat("trend________(20) . . . . . . Determine linear trends", "\n")
  cat("wfldmean_____(21) . . . . . . Determine the weighted field mean", "\n")
  cat("year.anomaly_(22) . . . . . . Determine annual anomalies", "\n")
  cat("yearmean_____(23) . . . . . . Determine annual means", "\n")
  cat("yearsum______(24) . . . . . . Determine annual sums", "\n")
  cat("ymonmean_____(25) . . . . . . Determine multi-year monthly means", "\n")
  cat("ymonsum______(26) . . . . . . Determine multi-year monthly sums", "\n")
  cat("yseasmean____(27) . . . . . . Determine multi-year seasonal means", "\n")

  cat("\n","Your choice (1 to 27, or 0 for more operators): ",sep="", "\n")
  func <- scan(file="",what=integer(),n=1,quiet=TRUE)
  time <- as.numeric(format(Sys.time(), "%H%M%S"))

  if (func>47|func<0){
    cat("\n",func," = bad choice",sep="", "\n") 
    cat("If you would like to have more functions included contact:","\n")
    cat("cmsaf.training@dwd.de","\n")
    break
  }
  
  if (func==0){
    cat( "\n","cmsaf.addc_____(28) . . . . . . Add a constant to a dataset",sep="", "\n")
    cat("cmsaf.divc_____(29) . . . . . . Divide data by a constant", "\n")
    cat("cmsaf.subc_____(30) . . . . . . Subtract a constant from a dataset", "\n")
    cat("dayrange_______(31) . . . . . . Determine the diurnal range", "\n")
    cat("extract.period_(32) . . . . . . Remove a time period", "\n")
    cat("monmax_________(33) . . . . . . Determine monthly maxima", "\n")
    cat("monmin_________(34) . . . . . . Determine monthly minima", "\n")
    cat("monsd__________(35) . . . . . . Determine monthly standard deviation", "\n")
    cat("remapbil_______(36) . . . . . . Bilinear grid interpolation", "\n")
    cat("timmax_________(37) . . . . . . Determine all-time maxima", "\n")
    cat("timmin_________(38) . . . . . . Determine all-time minima", "\n")
    cat("timpctl________(39) . . . . . . Determine percentile over all timesteps", "\n")
    cat("timsd__________(40) . . . . . . Determine all-time standard deviation", "\n")
    cat("timsum_________(41) . . . . . . Determine all-time sum", "\n")
    cat("ydaymean_______(42) . . . . . . Determine multi-year daily means", "\n")
    cat("ymonmax________(43) . . . . . . Determine multi-year monthly maxima", "\n")
    cat("ymonmin________(44) . . . . . . Determine multi-year monthly minima", "\n")
    cat("ymonsd_________(45) . . . . . . Determine multi-year monthly standard deviation", "\n")
    cat("yseasmax_______(46) . . . . . . Determine multi-year seasonal maxima", "\n")
    cat("yseasmin_______(47) . . . . . . Determine multi-year seasonal minima", "\n")
    cat("yseassd________(48) . . . . . . Determine multi-year seasonal standard deviation", "\n")
    
    cat("\n","Your choice (1 to 48): ",sep="", "\n")
    func <- scan(file="",what=integer(),n=1,quiet=TRUE)  
  }
  
  if (func>49|func<0){
    cat("\n",func," = bad choice",sep="", "\n") 
    cat("If you would like to have more functions included contact:","\n")
    cat("cmsaf.training@dwd.de","\n")
    break
  }

  if (func==1){
    cat("Multiply your data with a constant number:","\n")
    outfile <- paste(var,"_Apply.Function_cmsaf.mulc_",time,".nc",sep="")
    cat("Please enter a number: ", "\n")
    const <- scan(file="",what=double(),n=1,quiet=TRUE)
    cmsaf.mulc(var,const,infile,outfile,nc34=nc34)}
  
  if (func==2){
    cat("Divide by days per month:","\n")
    outfile <- paste(var,"_Apply.Function_divdpm_",time,".nc",sep="")
    divdpm(var,infile,outfile,nc34=nc34)}

  if (func==3){
   cat("Determine the field maximum:","\n")
   outfile <- paste(var,"_Apply.Function_fldmax_",time,".nc",sep="")  
   fldmax(var,infile,outfile,nc34=nc34)}

  if (func==4){
   cat("Determine the field mean:","\n")  
   outfile <- paste(var,"_Apply.Function_fldmean_",time,".nc",sep="")
   fldmean(var,infile,outfile,nc34=nc34)}

  if (func==5){
   cat("Determine the field minimum:","\n")  
   outfile <- paste(var,"_Apply.Function_fldmin_",time,".nc",sep="")
   fldmin(var,infile,outfile,nc34=nc34)}

  if (func==6){
   cat("Determine monthly anomalies:","\n")  
   outfile <- paste(var,"_Apply.Function_mon.anomaly_",time,".nc",sep="")
   mon.anomaly(var,infile,outfile,nc34=nc34)}

  if (func==7){
   cat("Determine monthly means:","\n")
   outfile <- paste(var,"_Apply.Function_monmean_",time,".nc",sep="")
   monmean(var,infile,outfile,nc34=nc34)}

  if (func==8){
   cat("Determine monthly sums:","\n")
   outfile <- paste(var,"_Apply.Function_monsum_",time,".nc",sep="")
   monsum(var,infile,outfile,nc34=nc34)}

  if (func==9){
   cat("Determine multi-monthly means:","\n")
   cat("Please enter months (1-12) as comma separated list (e.g., 7,8,9): ", "\n")
   month <- readline()
   month <- strsplit(month,",")
   month <- as.vector(as.numeric(unlist(month)))
   outfile <- paste(var,"_Apply.Function_multimonmean_",time,".nc",sep="")
   multimonmean(var,month,infile,outfile,nc34=nc34)}

  if (func==10){
   cat("Determine multi-monthly sums:","\n")
   cat("Please enter months (1-12) as comma separated list (e.g., 7,8,9): ", "\n")
   month <- readline()
   month <- strsplit(month,",")
   month <- as.vector(as.numeric(unlist(month)))
   outfile <- paste(var,"_Apply.Function_multimonsum_",time,".nc",sep="")
   multimonsum(var,month,infile,outfile,nc34=nc34)}

  if (func==11){
   cat("Multiply by days per month:","\n")
   outfile <- paste(var,"_Apply.Function_muldpm_",time,".nc",sep="")
   muldpm(var,infile,outfile,nc34=nc34)}

  if (func==12){
   cat("Determine seasonal anomalies:","\n")
   outfile <- paste(var,"_Apply.Function_seas.anomaly_",time,".nc",sep="")
   seas.anomaly(var,infile,outfile,nc34=nc34)}

  if (func==13){
   cat("Determine seasonal means:","\n")
   outfile <- paste(var,"_Apply.Function_seasmean_",time,".nc",sep="")
   seasmean(var,infile,outfile,nc34=nc34)}

  if (func==14){
    cat("Select a region:","\n")
    cat("Please enter minimum longitude: ", "\n")
    lon1 <- scan(file="",what=double(),n=1,quiet=TRUE)
    cat("Please enter maximum longitude: ", "\n")
    lon2 <- scan(file="",what=double(),n=1,quiet=TRUE)
    cat("Please enter minimum latitude: ", "\n")
    lat1 <- scan(file="",what=double(),n=1,quiet=TRUE)
    cat("Please enter maximum latitude: ", "\n")
    lat2 <- scan(file="",what=double(),n=1,quiet=TRUE)
    outfile <- paste(var,"_Apply.Function_sellonlatbox_",time,".nc",sep="")
    (sellonlatbox(var,infile,outfile,lon1,lon2,lat1,lat2,nc34=nc34))
  }

 if (func==15){
    cat("Extract a list of months:","\n")
    cat("Please enter month(s) (1-12) as comma separated list (e.g., 1,2,12): ", "\n")
    month <- readline()
    month <- strsplit(month,",")
    month <- as.vector(as.numeric(unlist(month)))
    outfile <- paste(var,"_Apply.Function_selmon_",time,".nc",sep="")
    selmon(var,month,infile,outfile,nc34=nc34)
  }

  if (func==16){
    cat("Extract a time period:","\n")
    cat("Please enter start date in form of YYYY-MM-DD: ", "\n")
    start <- scan(file="",what=character(),n=1,quiet=TRUE)
    cat("Please enter end date in form of YYYY-MM-DD: ", "\n")
    end <- scan(file="",what=character(),n=1,quiet=TRUE)
    outfile <- paste(var,"_Apply.Function_selperiod_",start,"_",end,"_",time,".nc",sep="")
    selperiod(var,start,end,infile,outfile,nc34=nc34)
  }

  if (func==17){
    cat("Extract data at a given point:","\n")
    cat("Please enter longitude of point: ", "\n")
    lon3 <- scan(file="",what=double(),n=1,quiet=TRUE)
    cat("Please enter latitude of point: ", "\n")
    lat3 <- scan(file="",what=double(),n=1,quiet=TRUE)
    outfile <- paste(var,"_Apply.Function_selpoint_",round(lon3),"_",round(lat3),"_",time,".nc",sep="")
    selpoint(var,infile,outfile,lon3,lat3,nc34=nc34)
  }

 if (func==18){
    cat("Extract a list of years:","\n")
    cat("Please enter year(s) as comma separated list (e.g., 1999,2003): ", "\n")
    year <- readline()
    year <- strsplit(year,",")
    year <- as.vector(as.numeric(unlist(year)))
    outfile <- paste(var,"_Apply.Function_selyear_",time,".nc",sep="")
    selyear(var,year,infile,outfile,nc34=nc34)
  }

  if (func==19){
   cat("Determine the overall mean:","\n")
   outfile <- paste(var,"_Apply.Function_timean_",time,".nc",sep="")
   timmean(var,infile,outfile,nc34=nc34)}

  if (func==20){
   cat("Determine the linear trend:","\n")
   cat("The cmsaf R-package (1.9.0 and higher) offers two options","\n")
   cat("to calculate the linear trend: option = 1 (fast, needs more","\n")
   cat("memory), option = 2 (slow, requires less memory).","\n")
   cat("Default is 1 (just hit Enter). If this fails, try option = 2.","\n")
   cat("Option 1 or 2? (1 or 2): ", "\n")
   trend_option <- scan(file="",what=integer(),n=1,quiet=TRUE)
   if(identical(trend_option, integer(0))){trend_option=1}
   outfile <- paste(var,"_Apply.Function_trend_",time,".nc",sep="")
   if (trend_option==2){
     trend(var,infile,outfile,option=2,nc34=nc34)
   } else {
      trend(var,infile,outfile,nc34=nc34)
     }
   }

  if (func==21){
   cat("Determine the weighted field mean:","\n")
   outfile <- paste(var,"_Apply.Function_wfldmean_",time,".nc",sep="")  
   wfldmean(var,infile,outfile,nc34=nc34)}

  if (func==22){
   cat("Determine annual anomalies:","\n")
   outfile <- paste(var,"_Apply.Function_year.anomaly_",time,".nc",sep="")
   year.anomaly(var,infile,outfile,nc34=nc34)}

  if (func==23){
   cat("Determine annual means:","\n")
   outfile <- paste(var,"_Apply.Function_yearmean_",time,".nc",sep="")
   yearmean(var,infile,outfile,nc34=nc34)}

  if (func==24){
   cat("Determine annual sums means:","\n")
   outfile <- paste(var,"_Apply.Function_yearsum_",time,".nc",sep="")
   yearsum(var,infile,outfile,nc34=nc34)}

  if (func==25){
   cat("Determine long-term monthly means:","\n")
   outfile <- paste(var,"_Apply.Function_ymonmean_",time,".nc",sep="")
   ymonmean(var,infile,outfile,nc34=nc34)}

  if (func==26){
   cat("Determine long-term monthly sums:","\n")
   outfile <- paste(var,"_Apply.Function_ymonsum_",time,".nc",sep="")
   ymonsum(var,infile,outfile,nc34=nc34)}

  if (func==27){
   cat("Determine long-term seasonal means:","\n")
   outfile <- paste(var,"_Apply.Function_yseasmean_",time,".nc",sep="")
   yseasmean(var,infile,outfile,nc34=nc34)}
  
  if (func==28){
    cat("Add a constant number to your data:","\n")
    outfile <- paste(var,"_Apply.Function_cmsaf.addc_",time,".nc",sep="")
    cat("Please enter a number: ", "\n")
    const <- scan(file="",what=double(),n=1,quiet=TRUE)
    cmsaf.addc(var,const,infile,outfile,nc34=nc34)}
  
  if (func==29){
    cat("Divide your data by a constant number:","\n")
    outfile <- paste(var,"_Apply.Function_cmsaf.divc_",time,".nc",sep="")
    cat("Please enter a number: ", "\n")
    const <- scan(file="",what=double(),n=1,quiet=TRUE)
    cmsaf.divc(var,const,infile,outfile,nc34=nc34)}
  
  if (func==30){
    cat("Subtract a constant number from your data:","\n")
    outfile <- paste(var,"_Apply.Function_cmsaf.subc_",time,".nc",sep="")
    cat("Please enter a number: ", "\n")
    const <- scan(file="",what=double(),n=1,quiet=TRUE)
    cmsaf.subc(var,const,infile,outfile,nc34=nc34)}
  
  if (func==31){
    cat("Determine the diurnal range from instantaneous or hourly data:","\n")
    outfile <- paste(var,"_Apply.Function_dayrange_",time,".nc",sep="")
    dayrange(var,infile,outfile,nc34=nc34)}
  
  if (func==32){
    cat("Remove a time period:","\n")
    cat("Please enter start date in form of 'YYYY-MM-DD': ", "\n")
    start <- scan(file="",what=character(),n=1,quiet=TRUE)
    cat("Please enter end date in form of 'YYYY-MM-DD': ", "\n")
    end <- scan(file="",what=character(),n=1,quiet=TRUE)
    outfile <- paste(var,"_Apply.Function_extract.period_",start,"_",end,"_",time,".nc",sep="")
    extract.period(var,start,end,infile,outfile,nc34=nc34)
  }
  
  if (func==33){
    cat("Determine monthly maxima:","\n")
    outfile <- paste(var,"_Apply.Function_monmax_",time,".nc",sep="")
    monmax(var,infile,outfile,nc34=nc34)}
  
  if (func==34){
    cat("Determine monthly minima:","\n")
    outfile <- paste(var,"_Apply.Function_monmin_",time,".nc",sep="")
    monmin(var,infile,outfile,nc34=nc34)}
  
  if (func==35){
    cat("Determine monthly standard deviation:","\n")
    outfile <- paste(var,"_Apply.Function_monsd_",time,".nc",sep="")
    monsd(var,infile,outfile,nc34=nc34)}
  
  if (func==36){
    cat("Bilinear grid interpolation:","\n")
    cat("Please choose the file with the target grid: ", "\n")
    infile2 <- file.choose()
    outfile <- paste(var,"_Apply.Function_remapbil_",time,".nc",sep="")
    remapbil(var,infile,infile2,outfile,nc34=nc34)
  }
  
  if (func==37){
    cat("Determine the maxima of all timesteps:","\n")
    outfile <- paste(var,"_Apply.Function_timmax_",time,".nc",sep="")
    timmax(var,infile,outfile,nc34=nc34)}
  
  if (func==38){
    cat("Determine the minima of all timesteps:","\n")
    outfile <- paste(var,"_Apply.Function_timmin_",time,".nc",sep="")
    timmin(var,infile,outfile,nc34=nc34)}
  
  if (func==39){
    cat("Determine the percentile over all timesteps:","\n")
    cat("Percentile number given as probability within [0,1]. Default is 0.95","\n")
    cat("Please enter a percentile number: ", "\n")
    const <- scan(file="",what=double(),n=1,quiet=TRUE)
    outfile <- paste(var,"_Apply.Function_timpctl_",time,".nc",sep="")
    timpctl(var,const,infile,outfile,nc34=nc34)}
  
  if (func==40){
    cat("Determine the standard deviation of all timesteps:","\n")
    outfile <- paste(var,"_Apply.Function_timsd_",time,".nc",sep="")
    timsd(var,infile,outfile,nc34=nc34)}
  
  if (func==41){
    cat("Determine the all-time sum:","\n")
    outfile <- paste(var,"_Apply.Function_timsum_",time,".nc",sep="")
    timsum(var,infile,outfile,nc34=nc34)}
  
  if (func==42){
    cat("Determine multi-year daily means:","\n")
    outfile <- paste(var,"_Apply.Function_ydaymean_",time,".nc",sep="")
    ydaymean(var,infile,outfile,nc34=nc34)}
  
  if (func==43){
    cat("Determine the multi-year monthly maxima:","\n")
    outfile <- paste(var,"_Apply.Function_ymonmax_",time,".nc",sep="")
    ymonmax(var,infile,outfile,nc34=nc34)}
  
  if (func==44){
    cat("Determine the multi-year monthly minima:","\n")
    outfile <- paste(var,"_Apply.Function_ymonmin_",time,".nc",sep="")
    ymonmin(var,infile,outfile,nc34=nc34)}
  
  if (func==45){
    cat("Determine the multi-year monthly standard deviation:","\n")
    outfile <- paste(var,"_Apply.Function_ymonsd_",time,".nc",sep="")
    ymonsd(var,infile,outfile,nc34=nc34)}
  
  if (func==46){
    cat("Determine the multi-year seasonal maxima:","\n")
    outfile <- paste(var,"_Apply.Function_yseasmax_",time,".nc",sep="")
    yseasmax(var,infile,outfile,nc34=nc34)}
  
  if (func==47){
    cat("Determine the multi-year seasonal minima:","\n")
    outfile <- paste(var,"_Apply.Function_yseasmin_",time,".nc",sep="")
    yseasmin(var,infile,outfile,nc34=nc34)}
  
  if (func==48){
    cat("Determine the multi-year seasonal standard deviation:","\n")
    outfile <- paste(var,"_Apply.Function_yseassd_",time,".nc",sep="")
    yseassd(var,infile,outfile,nc34=nc34)}

  cat("Do you want to apply another function? (y/n): ", "\n")
  go_on <- scan(file="",what=character(),n=1,quiet=TRUE)
  
  if(toupper(go_on)=="Y"){
    cat("Apply function to 1 or 2 ?:", "\n")
    cat(paste("1: ",outfile,sep=""), "\n")
    cat(paste("2: ",basename(infile_org),sep=""), "\n")
    apply2 <- scan(file="",what=integer(),n=1,quiet=TRUE)
    if(apply2==1){infile <- outfile}
    if(apply2==2)(infile <- infile_org)
    if(apply2!=1&apply2!=2) {
      cat(paste("1 or 2 not ",apply2,sep=""),"\n")
      infile <- outfile
    }
    cat(paste("The function will be applied to: ",infile,sep=""),"\n")
  } else {
      infile <- outfile
    }
}
  outdir <- file.path(ptt,"output",outfile)
  cat("Your final results are in:", "\n")
  cat(outdir, "\n")
  cat("\n","Do you want to display the results? (y/n): ",sep="", "\n")
  show_res <- scan(file="",what=character(),n=1,quiet=TRUE)
  
  if (toupper(show_res)=="Y"){
     infile <- outdir
     
     if (func==20){
      # Select variable

      id <- nc_open(infile)
        vn <- names(id$var)
      nc_close(id)
  
      var_default <- subset(vn, !(vn %in% c("lat","lon","time_bnds","nb2","time")))

      if (length(var_default)==1){
        var <- var_default
        cat("\n")
        cat("Variable ",var," will be used.",sep="","\n")
      } else {
          cat("\n")
          cat("Please enter variable (or push ENTER): ", "\n")
          var <- scan(file="",what=character(),n=1,quiet=TRUE)
          if(identical(var, character(0))){
            var <- var_default[1]
            cat("variable = ",var,sep="","\n")
          } else {
              if (toupper(var) %in% toupper(var_default)){
                var <- var_default[which(toupper(var)==toupper(var_default))]
              } else {
                  var <- var_default[1]
                  cat("Variable ",var," will be used.",sep="","\n")
                }
            }
        }
     } # end if func==20  

  # Load Libraries and data
    suppressMessages(library(shiny,warn.conflicts=FALSE,quietly=TRUE))
    suppressMessages(library(shinythemes,warn.conflicts=FALSE,quietly=TRUE))
    suppressMessages(library(RColorBrewer,warn.conflicts=FALSE,quietly=TRUE))
    suppressMessages(library(rworldmap,warn.conflicts=FALSE,quietly=TRUE))
    suppressMessages(library(rworldxtra,warn.conflicts=FALSE,quietly=TRUE))
    suppressMessages(library(colourpicker,warn.conflicts=FALSE,quietly=TRUE))
    suppressMessages(library(maps,warn.conflicts=FALSE,quietly=TRUE))
    suppressMessages(library(mapproj,warn.conflicts=FALSE,quietly=TRUE))
    data(countriesHigh)
    world <- as(countriesHigh,"SpatialLines")
    
    cat("To close the CM SAF R Visualizer click on the EXIT button.",sep="","\n")
    
    # set image default settings
    
    image_def <- 800         # default image size
    ihsf      <- 0.1         # default image heigth scale factor
    grid_col  <- "cornsilk2" # default color of grid lines
    plot_grid <- TRUE        # plot grid lines (TRUE = yes, FALSE = no)

    # Open file and plot fields
    id <- nc_open(infile)
      lon <- ncvar_get(id,"lon") 
      lat <- ncvar_get(id,"lat") 
      data <- ncvar_get(id,var,collapse_degen = F) 
      date <- ncvar_get(id,"time") 
      t_unit <- ncatt_get(id,"time","units")$value
      date.time <- as.character(get_time(t_unit,date))
      unit <- ncatt_get(id,var,"units")$value
      if (unit==0)(unit <- "-")
      varname <- ncatt_get(id,var,"long_name")$value
      if (varname==0)(varname <- ncatt_get(id,var,"standard_name")$value)
      if (varname==0)(varname <- var)
    nc_close(id)
    
  # In HOAPS files problems can occur due to different sorted values
      # check data dimensions
      if (dim(data)[1]!=length(lon)){
        if (dim(data)[2]==length(lon)){
          data <- aperm(data,c(2,1,3))
        }
      }
      # check longitude
      if (lon[1]>lon[length(lon)]){
        lon <- lon[length(lon):1]
        if (dim(data)[3]==1){
          data[,,1] <- data[length(lon):1,,]
        } else {
          data <- data[length(lon):1,,]
        }
      }
      # check latitude
      if (lat[1]>lat[length(lat)]){
        lat <- lat[length(lat):1]
        if (dim(data)[3]==1){
          data[,,1] <- data[,length(lat):1,]
        } else {
          data <- data[,length(lat):1,]
        }
      }

    if (length(lon)==1&length(lat)==1){
      min_data <- min(data,na.rm=T)
      max_data <- max(data,na.rm=T)
      x_range <- length(data)
      ltype <- c("l","p","o","s","h")
      date.time <- as.Date(date.time)
      fit <- lm(as.vector(data)~c(1:length(data)))
      dummy <- fit$fitted.values
      if(sum(!is.na(data))==length(dummy)){
        fitted <- data
        fitted[which(!is.na(fitted))] <- dummy
        rm(dummy)
      }
      ylabel <- paste(varname," [",unit,"]",sep="")
      if (imagewidth==-1&imageheight==-1){
        imageheight <- round(image_def*(2/3))
        imagewidth  <- image_def
      }
      runApp(file.path(ptt,"R-scripts/Toolbox_supp/App-4"))
    } else {
      min_data <- min(data,na.rm=T)
      max_data <- max(data,na.rm=T)
      
      if(round(min_data,digits=1)==round(max_data,digits=1)){
        min_data <- min_data-0.05
        max_data <- max_data+0.05
      }
  
      min_lon <- min(lon,na.rm=T)
      max_lon <- max(lon,na.rm=T)
  
      min_lat <- min(lat,na.rm=T)
      max_lat <- max(lat,na.rm=T)
      
      min_lon_slider <- min_lon-20
      max_lon_slider <- max_lon+20
      
      min_lat_slider <- min_lat-20
      max_lat_slider <- max_lat+20
      
      if(min_lon_slider<=-180)(min_lon_slider <- -180)
      if(min_lat_slider<=-90)(min_lat_slider <- -90)
      if(max_lon_slider>=180)(max_lon_slider <- 180)
      if(max_lat_slider>=90)(max_lat_slider <- 90)
      
      break_num <- function(ln,bn,minn,maxn){
        dg <- 2
        if(abs(max_data)>=10) (dg <- 1)
        if(abs(max_data)>=100)(dg <- 0)
        a <- vector(mode="character",length=bn)
        b <- round(seq(minn,maxn,length.out=bn),digits=dg)
        c <- round(seq(1,length(b),length.out=ln))
        a[c] <- b[c]
        labs <- a
        return(labs)
      }
      
      # get image height and width if not given
      if (imagewidth==-1&imageheight==-1){
        if (length(lon)>=length(lat)){
          imageheight <- round((image_def/length(lon))*length(lat))
          imageheight <- imageheight+(round((ihsf*imageheight)))
          imagewidth  <- image_def
        }
        if (length(lat)>length(lon)){
          imagewidth <- round((image_def/length(lon))*length(lat))
          imageheight  <- image_def
        }
      }
      runApp(file.path(ptt,"R-scripts/Toolbox_supp/App-3"))
    }
    
    } else {
      cat("Thanks for using the CM SAF R Toolbox.", "\n")
    }
  
  # clean up
    var_list_default <- c("apply2","bordercolor","break_num","checkstring","countriesHigh", 
                        "data","date","date.time","ddir","dummy","func","go_on","grid_col",
                        "i","id","ihsf","image_def","imageheight","imagewidth","infile",        
                        "infile_name","infile_org","lat","linesize","lon","max_data",
                        "max_lat","max_lat_slider","max_lon","max_lon_slider","min_data",
                        "min_lat","min_lat_slider","min_lon","min_lon_slider","n",
                        "na.color","outdir","outfile","plot_grid","show_res","slash",
                        "split_path","t","t_unit","textsize","time","unit","var",
                        "var_default","varname","vn","world","nc34")
  
    var_list <- ls()
  
    var_list <- var_list[var_list %in% var_list_default]
    var_list <- append(var_list,c("var_list_default","var_list"))
  
    rm(list=var_list)
  
  # end script Apply.Function.R
  
