# the CM SAF R TOOLBOX "Mostly Harmless"
# CM SAF R TOOLBOX control script
#
# Use this script to run the main parts of the CM SAF R TOOLBOX by one single script.
# You can also run all scripts on their own.
# 
#_________________________________________________________________________________
# Start user define section #
#
#
#   Optional path to CM SAF R TOOLBOX (no changes needed)
#   If ptt is not defined by the user, the path of the current script is taken
    ptt <- "NoNeedToChange"
#   e.g., ptt <- "C:/cmsaf/CMSAF-R-TOOLBOX"
#   default setting is: ptt <- "NoNeedToChange"
#
# Have fun with the CM SAF R TOOLBOX!         (Steffen Kothe / CM SAF 2018-07-18)
#________________________________________________________________________________
# End of user define section - No changes necessary beyond this point #





# Load Libraries

suppressMessages(library(cmsaf,warn.conflicts=FALSE,quietly=TRUE))    
suppressMessages(library(shiny,warn.conflicts=FALSE,quietly=TRUE))
suppressMessages(library(shinythemes,warn.conflicts=FALSE,quietly=TRUE))
suppressMessages(library(shinyFiles,warn.conflicts=FALSE,quietly=TRUE))

# Check ptt

if (ptt == "NoNeedToChange") {
  
  # Get path of current file
  
  slash <- "n"
  
  t <- try(ptt <- dirname(sys.frame(1)$ofile),TRUE)
  if("try-error" %in% class(t)|t==".") (ptt <- getwd())
  if (substr(ptt,1,1)=="/") (slash <- "t")
  
  # Get R-Toolbox path
  split_path <- function(path, mustWork = FALSE, rev = FALSE) {
    output <- c(strsplit(dirname(normalizePath(path,mustWork = mustWork)),
                         "/|\\\\")[[1]], basename(path))
    ifelse(rev, return(rev(output)), return(output))
  }
  dummy <- split_path(ptt)
  
  checkstring <- "CMSAF-R-TOOLBOX"
  n <- NULL
  
  for (i in 1:length(dummy)){
    if (grepl(checkstring,dummy[i])) n <- i
  }
  
  if (!is.null(n)){
    ptt <- do.call("file.path",c(as.list(dummy[1:n])))
    if (slash=="t") ptt <- file.path("/",ptt)
  }
} # end ptt check

# restart shiny application until exit button is clicked
repeat{
  value         <- 0
  selected_file <- "NoFile"
  if(exists("last_file")){
    last_file <- last_file
  } else {
    last_file     <- NULL
  }
  
  # launch main shiny control script
  
  if (value==0){
    value <- runApp(file.path(ptt,"R-scripts","Toolbox_supp","shinyApp_toolbox.R"))
  }
  
  if(is.character(value)){
    value <- as.numeric(value[1])
    selected_file <- "NoFile"
  } else {
    if(is.list(value)){
      selected_file <- as.character(value$datapath)
      value <- as.numeric(value[[1]][1])
    }
  }
  
  # remember last selected file
  if (selected_file != "NoFile"){
    last_file <- selected_file
  } 
  
  if(selected_file=="NoFile" & !is.null(last_file)){
    selected_file <- last_file
  }
  
  # evaluate return value and launch CM SAF R TOOLBOX scripts
  
  # launch Prep.Data.R
  if(value==1){
    source(file.path(ptt,"R-scripts","Prep.Data.R"))
  }
  
  # launch Apply.Function.R
  if(value==2){
    source(file.path(ptt,"R-scripts","Apply.Function.R"))
  }
  
  # launch CMSAF_Visulizer.R
  if(value==3){
    source(file.path(ptt,"R-scripts","CMSAF_Visualizer.R"))
  }
  
  if(value==99){
    cat("Thanks for using the CM SAF R Toolbox.\n")
    break
  }
}

# clean up
  var_list_default <- c("selected_file","value","checkstring","dummy","i",
                      "n","slash","split_path","t")
  var_list <- ls()

  var_list <- var_list[var_list %in% var_list_default]
  var_list <- append(var_list,c("var_list_default","var_list"))

  rm(list=var_list)

# end script Prep.Data.R

