# CM SAF R TOOLBOX NetCDF Data Visualizer
# The ability to save plots as png might depend on your firewall and the way
# of execution (in window or external).
#
# Have fun with the CM SAF R TOOLBOX!         (Steffen Kothe / CM SAF 2018-07-11)
#_________________________________________________________________________________
# Start user define section # 

  
  # Optional path to CM SAF R TOOLBOX (no changes needed)
  # If ptt is not defined by the user, the path of the current script is taken
    ptt <- "NoNeedToChange"
  # e.g., ptt <- "C:/cmsaf/CMSAF-R-TOOLBOX"
  # default setting is: ptt <- "NoNeedToChange"
  
  # Optional input (no changes needed)
    textsize    <- 1.2
    linesize    <- 1.5
    bordercolor <- "gray20"
    imagewidth  <- -1         # if -1 image dimensions are taken from data
    imageheight <- -1
    na.color    <- "gray80"
  
#_________________________________________________________________________________
# End of user define section - No changes necessary beyond this point #

  
  
  
    
# Load Libraries

  suppressMessages(library(cmsaf,warn.conflicts=FALSE,quietly=TRUE))    
  suppressMessages(library(shiny,warn.conflicts=FALSE,quietly=TRUE))
  suppressMessages(library(shinythemes,warn.conflicts=FALSE,quietly=TRUE))
  suppressMessages(library(shinyFiles,warn.conflicts=FALSE,quietly=TRUE))
  suppressMessages(library(rworldmap,warn.conflicts=FALSE,quietly=TRUE))
  suppressMessages(library(rworldxtra,warn.conflicts=FALSE,quietly=TRUE))
  suppressMessages(library(colourpicker,warn.conflicts=FALSE,quietly=TRUE))
  suppressMessages(library(maps,warn.conflicts=FALSE,quietly=TRUE))
  suppressMessages(library(mapproj,warn.conflicts=FALSE,quietly=TRUE))
  data(countriesHigh)
  world <- as(countriesHigh,"SpatialLines")

# set image default settings
  
  image_def <- 800         # default image size
  ihsf      <- 0.1         # default image heigth scale factor
  grid_col  <- "cornsilk2" # default color of grid lines
  plot_grid <- TRUE        # plot grid lines (TRUE = yes, FALSE = no)

# Check ptt
  
  if (ptt == "NoNeedToChange") {

    slash <- "n"
    
  # Get path of current file
    t <- try(ptt <- dirname(sys.frame(1)$ofile),TRUE)
    if("try-error" %in% class(t)|t==".") (ptt <- getwd())
    if (substr(ptt,1,1)=="/") slash <- "t"

  # Get R-Toolbox path
    split_path <- function(path) {setdiff(strsplit(path,"/|\\\\")[[1]],"")}
    dummy <- split_path(ptt)
  
    checkstring <- "CMSAF-R-TOOLBOX"
    n <- NULL
  
    for (i in 1:length(dummy)){
      if (grepl(checkstring,dummy[i])) n <- i
    }
  
    if (!is.null(n)){
      ptt <- do.call("file.path",c(as.list(dummy[1:n])))
      if (slash=="t") ptt <- file.path("/",ptt)
    
      # Test ptt
      if (!dir.exists(file.path(ptt,"data")) | !dir.exists(file.path(ptt,"figures")) |
        !dir.exists(file.path(ptt,"output"))) {
        stop(cat(paste("Something is wrong with your CMSAF-R-TOOLBOX path: ",ptt,sep="")),"\n")
      }
    }
  } # end ptt check

  cat("Your CMSAF-R-TOOLBOX path is: ",ptt,sep="","\n")

# Get infile name

   cat("Please enter infile name (e.g., 'SIS_monthly_means.nc')", "\n")
     infile_name <- as.character(NULL)
     setwd(file.path(ptt,"output"))
     if(exists("selected_file")){
       if(selected_file!="NoFile"){
         infile <- selected_file
       } else {
         infile <- file.choose()
       }
     } else {
       infile <- file.choose()
     }
     
     if(identical(infile, character(0))){
      stop(cat("No infile!",var,sep="","\n"))
     }  
  
  ncinfo(infile,"s")
  
# Select variable

  id <- nc_open(infile)
    vn <- names(id$var)
  nc_close(id)
  
  var_default <- subset(vn, !(vn %in% c("lat","lon","time_bnds","nb2","time")))
  
  # Stop if data are in sinusoidal projection
  
  if ("sinusoidal" %in% vn){
    sorry <- c("Sorry, the CM SAF R Toolbox can not handle data in sinusoidal projection.",
               "Please use the 'change projection' option during the order process.",
               "Your NetCDF data have to be on a regular lon-lat grid.")
    stop(writeLines(strwrap(sorry,width=80,indent=1)),call.=F)
  }
  
  ncinfo(infile,"s")
  
  if (length(var_default)==1){
    var <- var_default
    cat("\n")
    cat("Variable ",var," will be used.",sep="","\n")
  } else {
      cat("\n")
      cat("Please enter variable: ", "\n")
      var <- scan(file="",what=character(),n=1,quiet=TRUE)
      if(identical(var, character(0))){
        var <- var_default[1]
        cat("variable = ",var,sep="","\n")
      } else {
          if (toupper(var) %in% toupper(var_default)){
            var <- var_default[which(toupper(var)==toupper(var_default))]
          } else {
              var <- var_default[1]
              cat("Variable ",var," will be used.",sep="","\n")
            }
      }
  }
  
# Open file and get data
  id <- nc_open(infile)
    lon <- ncvar_get(id,"lon") 
    lat <- ncvar_get(id,"lat") 
    data <- ncvar_get(id,var,collapse_degen = F) 
    date <- ncvar_get(id,"time") 
    t_unit <- ncatt_get(id,"time","units")$value
    date.time <- as.character(get_time(t_unit,date))
    unit <- ncatt_get(id,var,"units")$value
    if (unit==0)(unit <- "-")
    varname <- ncatt_get(id,var,"long_name")$value
    if (varname==0)(varname <- ncatt_get(id,var,"standard_name")$value)
    if (varname==0)(varname <- var)
  nc_close(id)
  
  # In HOAPS files problems can occur due to different sorted values
      # check data dimensions
      if (dim(data)[1]!=length(lon)){
        if (dim(data)[2]==length(lon)){
          data <- aperm(data,c(2,1,3))
        }
      }
      # check longitude
      if (lon[1]>lon[length(lon)]){
        lon <- lon[length(lon):1]
        if (dim(data)[3]==1){
          data[,,1] <- data[length(lon):1,,]
        } else {
          data <- data[length(lon):1,,]
        }
      }
      # check latitude
      if (lat[1]>lat[length(lat)]){
        lat <- lat[length(lat):1]
        if (dim(data)[3]==1){
          data[,,1] <- data[,length(lat):1,]
        } else {
          data <- data[,length(lat):1,]
        }
      }
  
  cat("To close the CM SAF R Visualizer click on the EXIT button.",sep="","\n")
  
  if (length(lon)==1&length(lat)==1){
    min_data <- min(data,na.rm=T)
    max_data <- max(data,na.rm=T)
    x_range <- length(data)
    ltype <- c("l","p","o","s","h")
    date.time <- as.Date(date.time)
    fit <- lm(as.vector(data)~c(1:length(data)))
    dummy <- fit$fitted.values
    if(sum(!is.na(data))==length(dummy)){
      fitted <- data
      fitted[which(!is.na(fitted))] <- dummy
      rm(dummy)
    }
    ylabel <- paste(varname," [",unit,"]",sep="")
    if (imagewidth==-1&imageheight==-1){
      imageheight <- round(image_def*(2/3))
      imagewidth  <- image_def
    }
    runApp(file.path(ptt,"R-scripts/Toolbox_supp/App-4"))
  } else {
    min_data <- min(data,na.rm=T)
    max_data <- max(data,na.rm=T)
    
    if(round(min_data,digits=1)==round(max_data,digits=1)){
      min_data <- min_data-0.05
      max_data <- max_data+0.05
    }
  
    min_lon <- min(lon,na.rm=T)
    max_lon <- max(lon,na.rm=T)
  
    min_lat <- min(lat,na.rm=T)
    max_lat <- max(lat,na.rm=T)
    
    min_lon_slider <- min_lon-20
    max_lon_slider <- max_lon+20
    
    min_lat_slider <- min_lat-20
    max_lat_slider <- max_lat+20
    
    if(min_lon_slider<=-180)(min_lon_slider <- -180)
    if(min_lat_slider<=-90)(min_lat_slider <- -90)
    if(max_lon_slider>=180)(max_lon_slider <- 180)
    if(max_lat_slider>=90)(max_lat_slider <- 90)
  
    break_num <- function(ln,bn,minn,maxn){
      dg <- 2
      if(abs(max_data)>=10) (dg <- 1)
      if(abs(max_data)>=100)(dg <- 0)
      a <- vector(mode="character",length=bn)
      b <- round(seq(minn,maxn,length.out=bn),digits=dg)
      c <- round(seq(1,length(b),length.out=ln))
      a[c] <- b[c]
      labs <- a
      return(labs)
    }
    
    # get image height and width if not given
    if (imagewidth==-1&imageheight==-1){
      if (length(lon)>=length(lat)){
        imageheight <- round((image_def/length(lon))*length(lat))
        imageheight <- imageheight+(round((ihsf*imageheight)))
        imagewidth  <- image_def
      }
      if (length(lat)>length(lon)){
        imagewidth <- round((image_def/length(lon))*length(lat))
        imageheight  <- image_def
      }
    }
    runApp(file.path(ptt,"R-scripts/Toolbox_supp/App-3"))
  }
  
  # clean up
  var_list_default <- c("bordercolor","break_num","checkstring","countriesHigh","data",
                        "date","date.time","dummy","grid_col","i","id","ihsf","image_def",
                        "imageheight","imagewidth","infile","infile_name","lat","linesize",
                        "lon","max_data","max_lat","max_lat_slider","max_lon","max_lon_slider",
                        "min_data","min_lat","min_lat_slider","min_lon","min_lon_slider","n",
                        "na.color","plot_grid","slash","split_path","t","t_unit","textsize",
                        "unit","var","var_default","varname","vn","world")
  
  var_list <- ls()
  
  var_list <- var_list[var_list %in% var_list_default]
  var_list <- append(var_list,c("var_list_default","var_list"))
  
  rm(list=var_list)
  
  # end script CMSAF_Visualizer.R
  
