#
# This script will prepare CM SAF data using the 'cmsaf' R-package.
# The script will in a first step untar and unzip your ORD.tar file. 
# It does not check for incorrect inputs. Thus, be careful when you 
# enter options and check it before hitting enter. For usage of this 
# script 'cmsaf' R-package in version 1.8 or higher is recommended.  
# Have fun with the CM SAF R TOOLBOX!   
#                                            (Steffen Kothe / CM SAF 2018-08-06)
#_________________________________________________________________________________
# Start user define section #


  # Optional path to CM SAF R TOOLBOX (no changes needed)
  # If ptt is not defined by the user, the path of the current script is taken
    ptt <- "NoNeedToChange"
  # e.g., ptt <- "C:/cmsaf/CMSAF-R-TOOLBOX"
  # default setting is: ptt <- "NoNeedToChange"
  
  # The Windows-internal tar program can cause an issue 
  # If dates are NA, try to set tar_flag to 1
  # If problems occur, you should install wintar or Rtools
    
    tar_flag <- 0
    
  # Option for file output (no changes needed)
    nc34        <- 3          # output file format NetCDF3 or NetCDF4 (3 or 4)

#_________________________________________________________________________________
# End of user define section - No changes necessary beyond this point #





# Load libraries

  suppressMessages(library(R.utils,warn.conflicts=FALSE,quietly=TRUE))
  suppressMessages(library(cmsaf,warn.conflicts=FALSE,quietly=TRUE))

# Check ptt
  
if (ptt == "NoNeedToChange") {

# Get path of current file
  
  slash <- "n"
  
  t <- try(ptt <- dirname(sys.frame(1)$ofile),TRUE)
  if("try-error" %in% class(t)|t==".") (ptt <- getwd())
  if (substr(ptt,1,1)=="/") (slash <- "t")

# Get R-Toolbox path
  split_path <- function(path, mustWork = FALSE, rev = FALSE) {
    output <- c(strsplit(dirname(normalizePath(path,mustWork = mustWork)),
                                             "/|\\\\")[[1]], basename(path))
    ifelse(rev, return(rev(output)), return(output))
  }
  dummy <- split_path(ptt)
  
  checkstring <- "CMSAF-R-TOOLBOX"
  n <- NULL
  
  for (i in 1:length(dummy)){
    if (grepl(checkstring,dummy[i])) n <- i
  }
  
  if (!is.null(n)){
    ptt <- do.call("file.path",c(as.list(dummy[1:n])))
    if (slash=="t") ptt <- file.path("/",ptt)
  }
} # end ptt check
  
  # Test ptt
  if (nchar(ptt)>260)(warning("Your CMSAF-R-TOOLBOX path might be too long!"))
  
  if (!dir.exists(file.path(ptt,"data")) | !dir.exists(file.path(ptt,"figures")) |
    !dir.exists(file.path(ptt,"output"))) {
    stop(cat(paste("Something is wrong with your CMSAF-R-TOOLBOX path: ",ptt,sep="")),"\n")
  }

cat("Your CMSAF-R-TOOLBOX path is: ",ptt,sep="","\n")

# Choose tar-file

  cat("\n")
  cat("Please choose a tar-file (e.g., 'ORD12345.tar')", "\n")
  infile_name <- as.character(NULL)
  setwd(file.path(ptt,"data"))
  if(exists("selected_file")){
    if(selected_file!="NoFile"){
      infile_name <- selected_file
    } else {
      infile_name <- file.choose()
    }
  } else {
    infile_name <- file.choose()
  }
  
  ordname <- basename(infile_name)
  ordpath <- dirname(infile_name)
  if(identical(infile_name, character(0))){
    stop(cat("No infile!",var,sep="","\n"))
  }
  
  cat("\n")

# Check if there is more than one part of the order
  
  datadir <- file.path(ptt,"data")
  
  flist <- list.files(ordpath,substr(ordname,1,8))
  tarlist_all <- NULL
  for (i in 1:length(flist)){
    orddir <- file.path(ordpath,flist[i])
    if (tar_flag==1){
      tarlist_all <- append(tarlist_all,untar(orddir,list=TRUE,tar="internal"))
    }
    if (tar_flag==2){
      tarlist_all <- append(tarlist_all,untar(orddir,list=TRUE,tar=Sys.getenv("TAR")))
    }
    if (tar_flag==3){
      tarlist_all <- append(tarlist_all,untar(orddir,list=TRUE,tar="C:/Rtools/bin/tar.exe"))
    }
    if(tar_flag!=1&tar_flag!=2) {
      tarlist_all <- append(tarlist_all,untar(orddir,list=TRUE))
    }
  }
  
  tarlist_all <- sort(tarlist_all)
  claas_flag  <- 0
  
  # Check for CLAAS auxiliary data
  if ("CM_SAF_CLAAS2_L2_AUX.nc" %in% tarlist_all){
    tarlist_all <- subset(tarlist_all, !(tarlist_all %in% c("CM_SAF_CLAAS2_L2_AUX.nc")))
    claas_flag <- 1
  }

  # Select time period
  timestep <- substr(tarlist_all[1],4,4)
  
  if (timestep=="m"){
    date_from <- as.character(min(as.numeric(substr(tarlist_all,6,11))))
    date_from <- paste(substr(date_from,1,4),"-",substr(date_from,5,6),sep="")
    date_to <- as.character(max(as.numeric(substr(tarlist_all,6,11))))
    date_to <- paste(substr(date_to,1,4),"-",substr(date_to,5,6),sep="")
    cat("Dates are ranging from ",date_from," to ",date_to,sep="", "\n")
    cat("Please enter startdate in format YYYY-MM (or push ENTER): ", "\n")
    startdate <- scan(file="",what=character(),n=1,quiet=TRUE)
    if(identical(startdate, character(0))){
      startdate <- date_from
      cat(startdate,"\n","\n")
    }
    if(nchar(startdate)>7){
      startdate <- substr(startdate,1,7)
      cat(startdate,"\n","\n")
    }
    startdate <- as.numeric(gsub("-","",startdate))
    cat("Please enter enddate in format YYYY-MM (or push ENTER): ", "\n")
    enddate <- scan(file="",what=character(),n=1,quiet=TRUE)
    if(identical(enddate, character(0))){
      enddate <- date_to
      cat(enddate,"\n","\n")
    }
    if(nchar(enddate)>7){
      enddate <- substr(enddate,1,7)
      cat(enddate,"\n","\n")
    }
    enddate <- as.numeric(gsub("-","",enddate))
    dates_all <- as.numeric(substr(tarlist_all,6,11))
    if (!(is.numeric(startdate))) startdate <- min(as.numeric(substr(tarlist_all,6,11)))
    if (!(is.numeric(enddate))) enddate <- max(as.numeric(substr(tarlist_all,6,11)))
  } else {
    date_from <- as.character(min(as.numeric(substr(tarlist_all,6,13))))
    date_from <- paste(substr(date_from,1,4),"-",substr(date_from,5,6),"-",substr(date_from,7,8),sep="")
    date_to <- as.character(max(as.numeric(substr(tarlist_all,6,13))))
    date_to <- paste(substr(date_to,1,4),"-",substr(date_to,5,6),"-",substr(date_to,7,8),sep="")
    cat("Dates are ranging from ",date_from," to ",date_to,sep="", "\n")
    cat("Please enter startdate in format YYYY-MM-DD (or push ENTER): ", "\n")
    startdate <- scan(file="",what=character(),n=1,quiet=TRUE)
    if(identical(startdate, character(0))){
      startdate <- date_from
      cat(startdate,"\n","\n")  
    }
    if(nchar(startdate)==7){
      startdate <- paste(startdate,"-01",sep="")
    }
    startdate <- as.numeric(gsub("-","",startdate))
    cat("Please enter enddate in format YYYY-MM-DD (or push ENTER): ", "\n")
    enddate <- scan(file="",what=character(),n=1,quiet=TRUE)
    if(identical(enddate, character(0))){
      enddate <- date_to
      cat(enddate,"\n","\n")}
    if(nchar(enddate)==7){
      enddate <- paste(enddate,"-01",sep="")
    }
    enddate <- as.numeric(gsub("-","",enddate))
    dates_all <- as.numeric(substr(tarlist_all,6,13))
    if (!(is.numeric(startdate))) startdate <- min(as.numeric(substr(tarlist_all,6,13)))
    if (!(is.numeric(enddate))) enddate <- max(as.numeric(substr(tarlist_all,6,13)))
  }

  filelist <- NULL
  
  for (i in 1:length(flist)){ 
  
    # Define data directories

    orddir <- file.path(ordpath,flist[i])
    
    # Select time period and untar the compressed data file into data directory

    if (tar_flag==1){
      tarlist <- untar(orddir,list=TRUE,tar="internal")
    } else {
      tarlist <- untar(orddir,list=TRUE)
    }
    
     # Check for CLAAS auxiliary data
    if ("CM_SAF_CLAAS2_L2_AUX.nc" %in% tarlist){
      tarlist <- subset(tarlist, !(tarlist %in% c("CM_SAF_CLAAS2_L2_AUX.nc")))
      claas_flag <- 1
    }
    tarlist <- sort(tarlist)
    
    if (timestep=="m"){
      dates <- as.numeric(substr(tarlist,6,11))
    } else {
      dates <- as.numeric(substr(tarlist,6,13))
    }
  
    dums <- NA
    dume <- NA
    
    if (startdate %in% dates){
      dums <- which(dates==startdate)
    } else {
        if (startdate<dates[1]&enddate>=dates[1])(dums <- 1)
    }

    if (enddate %in% dates){
      dume <- which(dates==enddate)
    } else {
        if (enddate>dates[length(dates)])(dume <- length(dates))
    }

    dums <- dums[1]
    dume <- dume[length(dume)]
    
    if (!is.na(dums)){
      start <- dums
      end <- dume
  
      if (length(start)>1){
        start <- start[1]
        dums <- start}
      if (length(end)>1){
        end <- end[length(end)]
        dume <- end}

      if (dums>dume){
        start <- dume
        end <- dums
      }
  
    tarlist <- tarlist[start:end]
    filelist <- append(filelist,tarlist)
    cat("Untar and unzip data...","\n")
    if (length(tarlist)>2000){
      dum <- seq(2000,length(tarlist),2000)
      if (length(dum)==1){
        tarlist_alt <- tarlist[1:2000]
        if (tar_flag==1){
          untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip",tar="internal")
        } else {
          untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip")
        }
        tarlist_alt <- tarlist[(dum+1):length(tarlist)]
        if (tar_flag==1){
          untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip",tar="internal")
        } else {
          untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip")
        }
      }
      if (length(dum)>=2){
        tarlist_alt <- tarlist[1:2000]
        if (tar_flag==1){
          untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip",tar="internal")
        } else {
          untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip")
        }
        for (j in 1:(length(dum)-1)){
          tarlist_alt <- tarlist[(dum[j]+1):dum[j+1]]
          if (tar_flag==1){
            untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip",tar="internal")
          } else {
            untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip")
          }
        }
        tarlist_alt <- tarlist[(dum[j+1]+1):length(tarlist)]
        if (tar_flag==1){
          untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip",tar="internal")
        } else {
          untar(orddir, files=tarlist_alt, exdir=datadir, compressed="gzip")
        }
      }
    } else {
      if (tar_flag==1){
        untar(orddir, files=tarlist, exdir=datadir, compressed="gzip",tar="internal")
      } else {
        untar(orddir, files=tarlist, exdir=datadir, compressed="gzip")
      }
    }
  } # end if !is.na(dums) 
} # end loop length(flist)
  
# Untar auxiliary data if included
  
  if (claas_flag==1){
    orddir <- file.path(ordpath,flist[1])
    if (tar_flag==1){
      untar(orddir, files="CM_SAF_CLAAS2_L2_AUX.nc", exdir=datadir, compressed="gzip",tar="internal")
    } else {
        untar(orddir, files="CM_SAF_CLAAS2_L2_AUX.nc", exdir=datadir, compressed="gzip")
      }
  }

# Unzip data in case of NetCDF3
  
  gzcheck <- length(unlist(strsplit(tarlist_all[1],".nc")))

  if (gzcheck==2){
    for (i in 1:length(filelist)){
      zipfile <- file.path(datadir,filelist[i])
      gunzip(zipfile, overwrite=TRUE)
    }

# Get information of first file

    infile <- unlist(strsplit(filelist[1],".gz"))
  } else {infile <- filelist[1]}
  infile <- file.path(datadir,infile)

# Select variable and region
  
  id <- nc_open(infile)
    vn <- names(id$var)
    dn <- names(id$dim)
    lon_var <- "lon"
    lat_var <- "lat"
    
    '%ni%' <- Negate('%in%')
    
    if("lon" %ni% dn){
      if("longitude" %in% dn){
        lon_var <- "longitude"
      }
    }
    
    if("lat" %ni% dn){
      if("latitude" %in% dn){
        lat_var <- "latitude"
      }
    }

    if(lon_var %in% dn){
      lon_range <- range(ncvar_get(id,lon_var))
    }
    if(lat_var %in% dn){
      lat_range <- range(ncvar_get(id,lat_var))
    }
  nc_close(id)
  
  var_default <- subset(vn, !(vn %in% c("lat","lon","time_bnds","nb2","time","SATID",
                                        "latitude","longitude")))
  
  # Stop if data are in sinusoidal projection
  
  if ("sinusoidal" %in% vn){
    sorry <- c("Sorry, the CM SAF R Toolbox can not handle data in sinusoidal projection.",
               "Please use the 'change projection' option during the order process.",
               "Your NetCDF data have to be on a regular lon-lat grid.")
    file.remove(file.path(datadir,filelist))
    stop(writeLines(strwrap(sorry,width=80,indent=1)),call.=F)
  }
  
  ncinfo(infile,"s")
  
  if (length(var_default)==1){
    var <- var_default
    cat("\n")
    cat("Variable ",var," will be used.",sep="","\n")
  } else {
      cat("\n")
      cat("Please enter variable (or push ENTER): ", "\n")
      var <- scan(file="",what=character(),n=1,quiet=TRUE)
      if(identical(var, character(0))){
        var <- var_default[1]
        cat("variable = ",var,sep="","\n")
      } else {
          if (toupper(var) %in% toupper(var_default)){
            var <- var_default[which(toupper(var)==toupper(var_default))]
          } else {
              var <- var_default[1]
              cat("Variable ",var," will be used.",sep="","\n")
            }
      }
  }

  id <- nc_open(infile)
    sn <- ncatt_get(id,var,"standard_name")
    sn <- sn$value
  nc_close(id)
  
  if(lon_var %in% dn & lat_var %in% dn){
    lo1 <- lon_range[1]-10
    lo2 <- lon_range[2]+10
    la1 <- lat_range[1]-10
    la2 <- lat_range[2]+10
    if(lo1<(-180))(lo1 <- -180)
    if(lo2>180)(lo2 <- 180)
    if(la1<(-90))(la1 <- -90)
    if(la2>90)(la2 <- 90)
    
    # plot preview of contained region
    image(seq(lo1,lo2,1),seq(la1,la2,1),
          array(0:1,c(length(seq(lo1,lo2,1)),length(seq(la1,la2,1)))),col="white",
          main="Preview of available spatial coverage", xlab="longitude",ylab="latitude")
    map("world",add=T,col="gray36")
    grid()
    rect(lon_range[1],lat_range[1],lon_range[2],lat_range[2],lwd=3,col="brown4",density=0)
    rect(lon_range[1],lat_range[1],lon_range[2],lat_range[2],lwd=0.5,
         angle = 36,col="brown4",density=60)
    mtext(sn)
  }
  
  cat("\n")  
  cat("Please enter minimum longitude (or push ENTER): ", "\n")
  lon1 <- scan(file="",what=double(),n=1,quiet=TRUE)
  cat("Please enter maximum longitude (or push ENTER): ", "\n")
  lon2 <- scan(file="",what=double(),n=1,quiet=TRUE)
  cat("Please enter minimum latitude (or push ENTER): ", "\n")
  lat1 <- scan(file="",what=double(),n=1,quiet=TRUE)
  cat("Please enter maximum latitude (or push ENTER): ", "\n")
  lat2 <- scan(file="",what=double(),n=1,quiet=TRUE)
  
  cat("\n")

  if(identical(lon1, numeric(0))){
    lon1 <- -180
  }
  if(identical(lon2, numeric(0))){
    lon2 <- 180
  }
  if(identical(lat1, numeric(0))){
    lat1 <- -90
  }
  if(identical(lat2, numeric(0))){
    lat2 <- 90
  }

  if(lon_var %in% dn & lat_var %in% dn){
    lo1b <- lon_range[1]
    lo2b <- lon_range[2]
    la1b <- lat_range[1]
    la2b <- lat_range[2]
    if(lo1b<lon1)(lo1b <- lon1)
    if(lo2b>lon2)(lo2b <- lon2)
    if(la1b<lat1)(la1b <- lat1)
    if(la2b>lat2)(la2b <- lat2)
    
    # plot preview of chosen region
    image(seq(lo1,lo2,1),seq(la1,la2,1),
          array(0:1,c(length(seq(lo1,lo2,1)),length(seq(la1,la2,1)))),col="white",
          main="Preview of chosen spatial coverage", xlab="longitude",ylab="latitude")
    map("world",add=T,col="gray36")
    grid()
    rect(lon_range[1],lat_range[1],lon_range[2],lat_range[2],lwd=1,col="gray50",density=0)
    rect(lon_range[1],lat_range[1],lon_range[2],lat_range[2],lwd=0.5,
         angle=36,col="gray50",density=60)
    rect(lo1b,la1b,lo2b,la2b,lwd=3,col="brown4",density=0)
    rect(lo1b,la1b,lo2b,la2b,lwd=0.5,angle=36,col="gray50",density=60)
    mtext(sn)
    
    # clean
    rm(list=c("lo1","lo2","la1","la2","lo1b","lo2b","la1b","la2b","lon_range","lat_range","dn"))
  }
    
  # Regrid AOD or CLAAS data
  
  if (claas_flag==1){
    cat("Data will be regridded to rectangular grid...", "\n")
    # Regrid first file 
    infile  <- filelist[1]
    infile  <- file.path(datadir,infile)
    auxfile <- file.path(datadir,"CM_SAF_CLAAS2_L2_AUX.nc")
    dxy <- 0.05     # Target grid resolution
    
    # load FNN library
    library(FNN)

    id <- nc_open(infile)
      rdata <- ncvar_get(id,var)
    nc_close(id)

    id <- nc_open(auxfile)
      lon_reg <- ncvar_get(id,lon_var)
      lat_reg <- ncvar_get(id,lat_var)
    nc_close(id)
    
    lon_org <- seq(-90.0,90.0,dxy)
    lat_org <- lon_org

    lon <- lon_org[which(lon_org>(min(lon_reg,na.rm=T)-dxy)&lon_org<(max(lon_reg,na.rm=T)+dxy))]
    lat <- lat_org[which(lat_org>(min(lat_reg,na.rm=T)-dxy)&lat_org<(max(lat_reg,na.rm=T)+dxy))]
    
    # FNN solution

    lon_reg   <- as.vector(lon_reg)
    lat_reg   <- as.vector(lat_reg)
    lon_reg2   <- lon_reg[!is.na(lon_reg)]
    lat_reg2   <- lat_reg[!is.na(lon_reg)]

	  fnn_a <- get.knnx(lon,lon_reg2,k=1)
    fnn_b <- get.knnx(lat,lat_reg2,k=1)
    
    filelist_claas <- NULL
    
    for (i in 1:length(filelist)){
      infile  <- filelist[i]
      infile  <- file.path(datadir,infile)
      
      id <- nc_open(infile)
        rdata <- as.vector(ncvar_get(id,var))
      nc_close(id)
      
      rdata2 <- rdata[!is.na(lon_reg)]
      arr_new <- array(NA,c(length(lon),length(lat)))
      arr_new[cbind(fnn_a$nn.index,fnn_b$nn.index)] <- rdata2
      
      # Put new data into file
  
      t_name <- "time"
      t_standard_name = "time"
      t_calendar = "standard"
   
      lat_name = "latitude"
      lat_standard_name = "latitude"
      lat_long_name = "latitude"
      lat_units = "degrees_north"
      lat_axis = "Y"

      lon_name = "longitude"
      lon_standard_name = "longitude"
      lon_long_name = "longitude"
      lon_units = "degrees_east"
      lon_axis = "X"
   
      info = "Created with the CM SAF R Toolbox." 
      var_prec="float"
      nc34 = 3
   
      if (nc34==4){
        nc_format <- as.logical(1)
        compression = 4
      } else {
          nc_format <- as.logical(0)
          compression = NA
        }
   
      v_missing_value <- -999
  
      target <- array(NA,c(length(lon),length(lat),1)) 
      target[,,1] <- arr_new
      target[is.na(target)] <- v_missing_value
   
      id <- nc_open(infile)
        v_units         <- ncatt_get(id,var,"units")$value
        v_standard_name <- ncatt_get(id,var,"standard_name")$value
        v_long_name     <- ncatt_get(id,var,"long_name")$value
        time1           <- ncvar_get(id,"time") 
        t_units         <- ncatt_get(id,"time","units")$value
      nc_close(id)
  
      outfile <- file.path(datadir,paste("RG_",filelist[i],sep=""))
    
      x <- ncdim_def(name="lon",units=lon_units,vals=lon)
      y <- ncdim_def(name="lat",units=lat_units,vals=lat)
      t <- ncdim_def(name="time",units=t_units,vals=time1,unlim=TRUE)
    
      var1 <- ncvar_def(name=var,units=v_units,dim=list(x,y,t),missval=v_missing_value,
                      prec=var_prec,compression=compression)
    
      vars <- list(var1)
    
      ncnew <- nc_create(outfile,vars,force_v4=nc_format)
      
        ncvar_put(ncnew,var1,target)
    
        ncatt_put(ncnew,var,"standard_name",v_standard_name,prec="text")
        ncatt_put(ncnew,var,"long_name",v_long_name,prec="text")

        ncatt_put(ncnew,"time","standard_name",t_standard_name,prec="text")
        ncatt_put(ncnew,"time","calendar",t_calendar,prec="text")

        ncatt_put(ncnew,"lon","standard_name",lon_standard_name,prec="text")
        ncatt_put(ncnew,"lon","long_name",lon_long_name,prec="text")
        ncatt_put(ncnew,"lon","axis",lon_axis,prec="text")

        ncatt_put(ncnew,"lat","standard_name",lat_standard_name,prec="text")
        ncatt_put(ncnew,"lat","long_name",lat_long_name,prec="text")
        ncatt_put(ncnew,"lat","axis",lat_axis,prec="text")

        ncatt_put(ncnew,0,"Info",info,prec="text")
        
      nc_close(ncnew)
      
      filelist_claas <- append(filelist_claas,paste("RG_",filelist[i],sep=""))
    } # end for filelist
    
    # Remove original files
    file.remove(file.path(datadir,filelist))
    file.remove(auxfile)
    filelist <- filelist_claas
  }
  
  if (unlist(strsplit(var,"_"))[1]=="HLW"|unlist(strsplit(var,"_"))[1]=="HSH"){
    cat("Please enter level: ", "\n")
  	level <- scan(file="",what=integer(),n=1,quiet=TRUE)
  	if(identical(level, numeric(0)))(level <- 1)
  
  	# levbox_mergetime

  	cat("levbox_mergetime", "\n")
  	pattern <- substr(filelist[1],1,5)
  	outfile <- paste(var,"_",startdate,"-",enddate,".nc",sep="")
  	outdir <- file.path(ptt,"output",outfile)
  	levbox_mergetime(var,level,datadir,pattern,outdir,lon1,lon2,lat1,lat2,nc34=nc34)  
  }	else {
  
# box_mergetime

    cat("box_mergetime", "\n")
    pattern <- substr(filelist[1],1,5)
    outfile <- paste(var,"_",startdate,"-",enddate,".nc",sep="")
    outdir <- file.path(ptt,"output",outfile)
    box_mergetime(var,datadir,pattern,outdir,lon1,lon2,lat1,lat2,nc34=nc34)
  }

  cat("\n")
  cat("You will find your merged data file here:", "\n")
  cat(file.path(ptt,"output",outfile), "\n")
  
  # remember outfile
    if(exists("last_file")){
      last_file <- file.path(ptt,"output",outfile) 
    }
  
# Clean up
  
  cat("\n","Delete extracted files? (recommended) (y/n): ",sep="")
  delete <- scan(file="",what=character(),n=1,quiet=TRUE)
  
  if (toupper(delete)!="N"){
    if (gzcheck==2)(filelist <- unlist(strsplit(filelist,".gz")))
      file.remove(file.path(datadir,filelist))
  } else {
      cat("Okay. Let's keep these files.","\n")
  }

  # Clean up variables
    var_list_default <- c("checkstring", "datadir", "date_from", "date_to", "dates", 
                        "dates_all","delete", "dume", "dummy", "dums", "end", "enddate", 
                        "filelist", "flist","gzcheck", "i", "id", "infile", "infile_name", 
                        "lat1", "lat2", "lon1","lon2", "n", "orddir", "ordname", "ordpath", 
                        "outdir", "outfile", "pattern","slash", "split_path", "start", 
                        "startdate", "tar_flag", "tarlist","tarlist_all", "timestep", "var", 
                        "var_default", "vn","zipfile","t","claas_flag","sn","nc34",
                        "lon_var","lat_var","%ni%")
    var_list <- ls()
  
    var_list <- var_list[var_list %in% var_list_default]
    var_list <- append(var_list,c("var_list_default","var_list"))
  
    rm(list=var_list)
  
  # end script Prep.Data.R
  