# (Steffen Kothe / CM SAF 2018-08-08)

shinyServer(function(input, output, session) {
  
  # Prepare R-Instat plotting
  plot_rinstat  <- reactiveVal(0)
  plot_rinstat2 <- reactiveVal(0)
  
  # depending on OS
  if(Sys.info()['sysname']=="Windows"){
    root = c('home' = '~','C:' = 'C:/','Users' = 'C:/Users/')
  } else {
    root = getVolumes()
  }
  
  shinyFileChoose(input,'file2',session=session,roots=root)
  
  instat.data <- reactive({
     if(!is.integer(input$file2)){
      fs <- parseFilePaths(root, input$file2)
      output$txt_file <- renderText(paste("Selected file: ",as.character(fs$datapath),sep=""))
      if(grepl("RData",as.character(fs$datapath))){
        a <- get(load(as.character(fs$datapath)))
        a <- as.data.frame(a)
        a
      }
    }
  })
  
  observeEvent(input$file2, {
     if(!is.integer(input$file2)){
      fs <- parseFilePaths(root, input$file2)
      output$txt_file <- renderText(paste("Selected file: ",as.character(fs$datapath),sep=""))
      if(grepl("RData",as.character(fs$datapath))){
        plot_rinstat(plot_rinstat()+1)
        plot_rinstat2(plot_rinstat2()+1)
      }
    }
  })
  
  observeEvent(input$instat,{
    if(input$instat==FALSE){
      plot_rinstat(plot_rinstat()*0)
    }
    if(input$instat==TRUE&plot_rinstat2()>0){
      plot_rinstat(plot_rinstat()*0)
      plot_rinstat(plot_rinstat()+1)
    }
  })
  
  # Combine the CM SAF R Toolbox and R-Instat
  co.data <- reactive({
    if(plot_rinstat()==1){
      # check row.names of data frame
      lo_dummy <- c("lon","longitude","laenge","x")
      la_dummy <- c("lat","latitude","breite","y")
      ti_dummy <- c("time","date","zeit","t")
      da_dummy <- c("data","daten","z","element")
      
      dn <- attr(instat.data(),"element_name")
      if(!is.null(dn)){
        da_dummy <- append(da_dummy,dn)
      } else {
          dn <- attr(instat.data(),"data_name")
          if(!is.null(dn)){
            da_dummy <- append(da_dummy,dn)
          }
        }
      
      instat_names <- names(instat.data())
      
      lo_n <- 0
      la_n <- 0
      ti_n <- 0
      da_n <- 0
      
      for (i in 1:length(instat_names)){
        if(toupper(instat_names[i]) %in% toupper(lo_dummy))(lo_n <- i)
        if(toupper(instat_names[i]) %in% toupper(la_dummy))(la_n <- i)
        if(toupper(instat_names[i]) %in% toupper(ti_dummy))(ti_n <- i)
        if(toupper(instat_names[i]) %in% toupper(da_dummy))(da_n <- i)
      }
      
      if(lo_n>0&la_n>0&ti_n>0&da_n>0){
        
        # check monthly or daily
        # station
        time_station <- instat.data()[,ti_n]
        if(length(time_station)>500)(time_station <- time_station[1:500])
        mon_station  <- format(as.Date(time_station), "%m")
        year_station <- format(as.Date(time_station), "%Y")
        day_station  <- format(as.Date(time_station), "%d")
        dummy <- which(mon_station==mon_station[1]&year_station==year_station[1])
        mmdm <- "d"
        if(length(unique(day_station[dummy]))==1){
          mmdm <- "m"
        }
        
        # satellite
        time_sat <- date.time
        if(length(time_sat)>40)(time_sat <- time_sat[1:40])
        mon_sat  <- format(as.Date(time_sat), "%m")
        year_sat <- format(as.Date(time_sat), "%Y")
        day_sat  <- format(as.Date(time_sat), "%d")
        dummy <- which(mon_sat==mon_sat[1]&year_sat==year_sat[1])
        mmdm_sat <- "d"
        if(length(unique(day_sat[dummy]))==1){
          mmdm_sat <- "m"
        }
        
        # extract data for chosen time step
        if(mmdm=="m"&mmdm_sat=="m"){
          match_time   <- which(format(as.Date(instat.data()[,ti_n]),"%Y-%m")==format(as.Date(input$timestep),"%Y-%m"),arr.ind = T)
        } else {
          match_time   <- which(instat.data()[,ti_n]==input$timestep,arr.ind = T)
        }
        
        lon_station  <- instat.data()[,lo_n][match_time]
        lat_station  <- instat.data()[,la_n][match_time]
        data_station <- instat.data()[,da_n][match_time]
        # delete NAs
        dummy <- !is.na(data_station)
        data_station <- data_station[dummy]
        data_station <- data_station
        lon_station  <- lon_station[dummy]
        lat_station  <- lat_station[dummy]
        # Extract corresponding data points
        data_sat <- c(1:length(data_station))
        
        for (istation in 1:length(data_station)){
          # find closest point to target coordinates using sp package
          
          dlon <- abs(lon[1]-lon[2])
          dlat <- abs(lat[1]-lat[2])
          
          lon_limit <- which(lon>=(lon_station[istation]-dlon)&lon<=(lon_station[istation]+dlon))  
          lat_limit <- which(lat>=(lat_station[istation]-dlat)&lat<=(lat_station[istation]+dlat)) 
          
          if (any(lon_limit)&any(lat_limit)){
            
            lon2 <- lon[lon_limit]
            lat2 <- lat[lat_limit]
            
            pos <- SpatialPoints(cbind(lon_station[istation],lat_station[istation]), proj4string=CRS("+proj=longlat +datum=WGS84"))
            dum_dist <- 1000
            for (i in 1:length(lon2)){
              for (j in 1:length(lat2)){
                dist <- spDistsN1(pos, c(lon2[i],lat2[j]), longlat = FALSE)
                if (dist<=dum_dist){
                  dum_dist <- dist
                  dumi <- i
                  dumj <- j
                }
              }
            }
            
            lon_limit <- which(lon==lon2[dumi])  
            lat_limit <- which(lat==lat2[dumj])
          }
          
          if (any(lon_limit)&any(lat_limit)){
            data_sat[istation] <- data[lon_limit,lat_limit,which(date.time==input$timestep,arr.ind = T)]
          }
        }
        cd <- data.frame(data_station,data_sat,lon_station,lat_station)
        cd
      } # end if lo_n,la_n,ti_n,da_n
    }
  })
  
  # prepare input for own locations
  lon_loc_vec  <- reactiveVal()
  lat_loc_vec  <- reactiveVal()
  name_loc_vec <- reactiveVal()
  
  observeEvent(input$add_loc,{
    lon_loc_vec(append(lon_loc_vec(),input$lon_loc))
    lat_loc_vec(append(lat_loc_vec(),input$lat_loc))
    name_loc_vec(append(name_loc_vec(),input$name_loc))
  })
  
  observeEvent(input$exit, {
    stopApp()
  })
  
  output$summary1 <- renderPrint({
    ncinfo(infile)
  })
  
  output$summary2 <- renderPrint({
    ncinfo(infile,"m")
  })
  
  
  output$about <- renderPrint({
    cat("The CMSAF Visualizer is part of the CM SAF R Toolbox.","\n")
    cat("The script helps you to visualize 1D-timeseries and 2D-maps.","\n")
    cat("Your NetCDF data have to follow CF conventions and the data","\n")
    cat("have to be on a rectangular grid with longitude, latitude","\n")
    cat("and time dimension.","\n")
    cat("This version (July 2018) was tested with the cmsaf R-package","\n")
    cat("in version 1.9.2.","\n")
    cat("\n")
    cat("Suggestions for improvements and praise for the developers","\n")
    cat("can be send to contact.cmsaf@dwd.de.","\n")
    cat("\n")
    cat("                              - Steffen Kothe - 2018-07-25 -","\n")
    cat("\n")
    cat("\n")
  })
  
  url <- a("http://www.cmsaf.eu/tools", href="http://www.cmsaf.eu/tools")
  output$link <- renderUI({
    tagList("URL link:", url)
  })
  
  output$tipps <- renderPrint({
    cat("When you save a figure, please add the file extension .png !","\n")
    cat("\n")
    cat("If saving an image fails, try right-click plus 'save image as'.","\n")
    cat("\n")
    cat("The orthographic projection includes some interpolation,","\n")
    cat("which can have an impact on local pixel values!","\n")
    cat("\n")
  })
  
  output$statistics <- renderPrint({
    # compute some statistics
    xr <- which(lon>=(input$slider1[1])&lon<=(input$slider1[2]))
    yr <- which(lat>=(input$slider2[1])&lat<=(input$slider2[2]))
    dastat <- data[xr,yr,which(date.time==input$timestep,arr.ind = T)]
    
    dg <- 2
    if(abs(max_data)>=10) (dg <- 1)
    if(abs(max_data)>=100)(dg <- 0)
    
    da_mean   <- round(mean(dastat,na.rm=T),digits=dg)
    da_median <- round(median(dastat,na.rm=T),digits=dg)
    da_sd     <- round(sd(dastat,na.rm=T),digits=dg)
    da_max    <- round(max(dastat,na.rm=T),digits=dg)
    da_min    <- round(min(dastat,na.rm=T),digits=dg)
    
    cat(paste("Mean:               ",da_mean,sep=""),"\n")
    cat(paste("Median:             ",da_median,sep=""),"\n")
    cat(paste("Standard deviation: ",da_sd,sep=""),"\n")
    cat(paste("Maximum:            ",da_max,sep=""),"\n")
    cat(paste("Minimum:            ",da_min,sep=""),"\n")
    cat(paste("Unit:               ",unit,sep=""),"\n")
  })
  
  output$myHist <- renderPlot({
    xr <- which(lon>=(input$slider1[1])&lon<=(input$slider1[2]))
    yr <- which(lat>=(input$slider2[1])&lat<=(input$slider2[2]))
    dastat <- data[xr,yr,which(date.time==input$timestep,arr.ind = T)]
    dastat <- as.numeric(dastat)
    
    hist(dastat,main=paste("Histogram of ",input$text1,sep=""),
         xlab=input$text3,col=rgb(91,127,149,maxColorValue = 255))
    rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], 
         col = "light grey")
    grid (NULL,NULL, lty = 3, col = grid_col, lwd=1.5)
    hist(dastat,col=rgb(91,127,149,maxColorValue = 255),add=T)
    box(col=bordercolor,lwd=linesize)
  })
  
  output$myComp <- renderPlot({
    if(plot_rinstat()==1){
      lo <- as.numeric(co.data()$lon_station)
      la <- as.numeric(co.data()$lat_station)
      st <- co.data()$data_station
      sa <- co.data()$data_sat
      st <- st[order(la)]
      sa <- sa[order(la)]
      lo <- lo[order(la)]
      la <- la[order(la)]
      xlabs <- NULL 
      for(i in 1:length(st)){
        dummy <- paste("[",round(lo[i],digit=1),
                       ";",round(la[i],digit=1),"]",sep="")
        xlabs <- append(xlabs,dummy)
      }
      rd <- rbind(st,sa)
      rownames(rd) <- c("R-Instat data","Your data")
      par(mar=c(6,5,3,2))
      barplot(rd,beside=T,main=paste("Comparison of ",input$text1,sep=""),
              ylab=input$text3,names.arg=xlabs,col=c(rgb(0,32,91,maxColorValue = 255),
                                                     rgb(242,169,0,maxColorValue = 255)),
              las=2)
      rect(par("usr")[1], par("usr")[3], par("usr")[2], par("usr")[4], 
           col = "light grey")
      grid (NULL,NULL, lty = 3, col = grid_col, lwd=1.5)
      barplot(rd,beside=T,
              ylab=input$text3,names.arg=xlabs,col=c(rgb(0,32,91,maxColorValue = 255),
                                                     rgb(242,169,0,maxColorValue = 255)),
              las=2,add=T,legend.text=c("Surface","Satellite"))
      box(col=bordercolor,lwd=linesize)
    }
  })
  
  output$myImage <- renderImage({
    # A temp file to save the output.
    # This file will be removed later by renderImage
    outfile <- tempfile(fileext='.png')
    
    # breaks 
    breaks <- seq(input$num_rmim,input$num_rmax,length.out=(input$num_tick))
    tlab <- break_num(input$num_tick,input$num_tick,input$num_rmim,input$num_rmax)
    xtick  <- axisTicks(input$slider1,log=F)
    ytick  <- axisTicks(input$slider2,log=F)
    xlab <- unlist(lapply(xtick, function(x) ifelse(x < 0, paste(abs(x), " W",sep=""), 
                                                    ifelse(x > 0, paste(abs(x), " E",sep=""),x))))
    ylab <- unlist(lapply(ytick, function(x) ifelse(x < 0, paste(abs(x), " S",sep=""), 
                                                    ifelse(x > 0, paste(abs(x), " N",sep=""),x))))
    
    if(min(xtick)==round(input$slider1[1])){xlab[1] <- " "}
    if(max(xtick)==round(input$slider1[2])){xlab[length(xlab)] <- " "}
    if(min(ytick)==round(input$slider2[1])){ylab[1] <- " "}
    if(max(ytick)==round(input$slider2[2])){ylab[length(ylab)] <- " "}
    
    # If rectangular projection
    if (input$proj == "rect"){
      
      # Generate the PNG with different color palettes
      if (input$checkGroup==1){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=tim.colors(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=input$text3,legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=tim.colors(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- tim.colors(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==2){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=heat.colors(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=heat.colors(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- heat.colors(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==3){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=terrain.colors(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=terrain.colors(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- terrain.colors(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==4){
        co <- colorRampPalette(c("darkblue","white","darkred"))
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co_ri <- co(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co_ri[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==5){
        co <- colorRampPalette(c("darkred","white","darkblue"))
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co_ri <- co(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co_ri[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==6){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=rainbow(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=rainbow(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- rainbow(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==7){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=cm.colors(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=cm.colors(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- cm.colors(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==8){
        co <- colorRampPalette(c("black","#3a0303","#640000","#981800","#ca4b00",
                                 "#fc7f01","#ffb234","#ffe566","#ffff98","#ffffcb","white"))
        iwidth  <- imagewidth
        iheight <- imageheight
        png(outfile, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co_ri <- co(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co_ri[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
    }
    
    # If orthographic projection
    if (input$proj == "ortho"){
      # prepare plot
      ori  <- c(input$xort,input$yort,input$rort)             #orientation
      nx <- length(lon)
      ny <- length(lat)
      landcol  <- "navajowhite3" 
      oceancol <- "cadetblue3"
      outcol   <- "cornsilk4"
      
      rep.row<-function(x,n){
        matrix(rep(x,each=n),nrow=n)
      }
      
      lonv  <- replicate(length(lat),lon)
      latv  <- rep.row(lat,length(lon))
      datav <- as.vector(data[,,which(date.time==input$timestep,arr.ind = T)])
      
      a <- mapproject(x=lonv,y=latv,projection="orthographic",orientation=ori)
      m <- map("world",plot=F)
      
      # filter Nas
      if (sum(is.na(a$x))>0|sum(is.na(a$y))>0){
        dummy <- NULL
        dummy <- !is.na(a$x)
        a$x   <- a$x[dummy]
        a$y   <- a$y[dummy]
        datav <- datav[dummy]
        dummy <- NULL
        dummy <- !is.na(a$y)
        a$x   <- a$x[dummy]
        a$y   <- a$y[dummy]
        datav <- datav[dummy]
      }
      
      # define grid factors
      xr <- abs(range(lon,na.rm=T)[1])+abs(range(lon,na.rm=T)[2])
      yr <- abs(range(lat,na.rm=T)[1])+abs(range(lat,na.rm=T)[2])
      l1 <- 3.1  # max value for nx/xf
      l2 <- 2.0  # max value for ny/yf
      
      x1 <- c(40,360)
      y1 <- c(1,l1)
      c1 <- lm(y1~x1)$coeff[[1]]
      c2 <- lm(y1~x1)$coeff[[2]]
      
      if (xr>40 & xr<=360){
        xf <- c2*xr+c1
        xf <- round(xf,digits=1)
      } else { xf <- 1}
      
      x1 <- c(40,180)
      y1 <- c(1,l2)
      c1 <- lm(y1~x1)$coeff[[1]]
      c2 <- lm(y1~x1)$coeff[[2]]
      
      if (yr>40 & yr<=180){
        yf <- c2*yr+c1
        yf <- round(yf,digits=1)
      } else { yf <- 1}
      
      # define imagesize for orthographic plot
      # if(imageheight>imagewidth){
      #   iwidth  <- imageheight
      #   iheight <- imageheight
      #   iheight <- iheight + (iheight*0.06)
      # } else {
      #   iwidth  <- imageheight
      #   iheight <- imageheight
      #   iheight <- iheight + (iheight*0.06)
      # }
      
      iwidth  <- 800
      iheight <- 800
      
      # define color palette
      if (input$checkGroup==1){pcol <- tim.colors(input$num_brk)}
      if (input$checkGroup==2){pcol <- heat.colors(input$num_brk)}
      if (input$checkGroup==3){pcol <- terrain.colors(input$num_brk)}
      if (input$checkGroup==4){
        co <- colorRampPalette(c("darkblue","white","darkred"))
        pcol <- co(input$num_brk)}
      if (input$checkGroup==5){
        co <- colorRampPalette(c("darkred","white","darkblue"))
        pcol <- co(input$num_brk)}
      if (input$checkGroup==6){pcol <- rainbow(input$num_brk)}
      if (input$checkGroup==7){pcol <- cm.colors(input$num_brk)}
      if (input$checkGroup==8){
        co <- colorRampPalette(c("black","#3a0303","#640000","#981800",
                                 "#ca4b00","#fc7f01","#ffb234","#ffe566",
                                 "#ffff98","#ffffcb","white"))
        pcol <- co(input$num_brk)}
      
      tlab <- break_num(input$num_tick,input$num_tick,input$num_rmim,input$num_rmax)  
      
      # Plot orthographic image
      png(outfile, width=iwidth, height=iheight)
      
      quilt.plot(a$x,a$y,datav,xlim=c(-1,1),ylim=c(-1,1),nx=nx/xf,ny=ny/yf,xlab=" ",ylab=" ",
                 main=input$text1,col=pcol,
                 axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                mgp=c(1,0.4,0), tck=c(-0.3)),
                 legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
      image(lon,lat,array(1:2,dim(data[,,which(date.time==input$timestep,arr.ind = T)]))
            ,xlab=" ",ylab=" ",col="black",axes=F,xlim=c(-1,1),ylim=c(-1,1),add=T)
      polygon(sin(seq(0,2*pi,length.out=100)),cos(seq(0,2*pi,length.out=100)), 
              col=oceancol, border=rgb(1,1,1,0.5), lwd=1)
      suppressWarnings(map("world",projection="orthographic",orientation=ori,add=T,interior=F
                           , fill=T, col=landcol,lwd=linesize,resolution=0,border=NA))
      quilt.plot(a$x,a$y,datav,xlim=c(-1,1),ylim=c(-1,1),nx=nx/xf,ny=ny/yf,xlab=" ",ylab=" ",
                 main=input$text1,col=pcol,
                 axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                mgp=c(1,0.4,0), tck=c(-0.3)),
                 legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F,add=T)
      if(!as.logical(input$int)){suppressWarnings(map("world",projection="orthographic",orientation=ori,add=T,interior=F
                                                      ,col=outcol,lwd=linesize,resolution=0))
      } else {
        suppressWarnings(map("world",projection="orthographic",orientation=ori,add=T,interior=T
                             ,col=bordercolor,lwd=linesize,resolution=0))
      }
      if(plot_grid){map.grid(m,nx=18,ny=9,lty=3,col=grid_col,cex=linesize)}
      mtext(input$text2)
      
      
      on.exit(dev.off())
    }
    
    # Return a list containing the filename
    list(src = outfile,
         contentType = 'image/png',
         width = iwidth,
         height = iheight,
         alt = "This is alternate text")
  }, deleteFile = TRUE)
  
  output$downloadPlot <- downloadHandler(
    filename = function(){
      paste(var,"_plot.png",sep="")},
    content = function(file) {
      
      # breaks 
      breaks <- seq(input$num_rmim,input$num_rmax,length.out=(input$num_tick))
      tlab <- break_num(input$num_tick,input$num_tick,input$num_rmim,input$num_rmax)
      xtick  <- axisTicks(input$slider1,log=F)
      ytick  <- axisTicks(input$slider2,log=F)
      xlab <- unlist(lapply(xtick, function(x) ifelse(x < 0, paste(abs(x), " W",sep=""), 
                                                      ifelse(x > 0, paste(abs(x), " E",sep=""),x))))
      ylab <- unlist(lapply(ytick, function(x) ifelse(x < 0, paste(abs(x), " S",sep=""), 
                                                      ifelse(x > 0, paste(abs(x), " N",sep=""),x))))
      
      if(min(xtick)==round(input$slider1[1])){xlab[1] <- " "}
      if(max(xtick)==round(input$slider1[2])){xlab[length(xlab)] <- " "}
      if(min(ytick)==round(input$slider2[1])){ylab[1] <- " "}
      if(max(ytick)==round(input$slider2[2])){ylab[length(ylab)] <- " "}
      
      if (input$checkGroup==1){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(file, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=tim.colors(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=tim.colors(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- tim.colors(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==2){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(file, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=heat.colors(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=heat.colors(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- heat.colors(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==3){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(file, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=terrain.colors(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=terrain.colors(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- terrain.colors(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==4){
        co <- colorRampPalette(c("darkblue","white","darkred"))
        iwidth  <- imagewidth
        iheight <- imageheight
        png(file, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co_ri <- co(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co_ri[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==5){
        co <- colorRampPalette(c("darkred","white","darkblue"))
        iwidth  <- imagewidth
        iheight <- imageheight
        png(file, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co_ri <- co(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co_ri[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==6){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(file, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=rainbow(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=rainbow(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- rainbow(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==7){
        iwidth  <- imagewidth
        iheight <- imageheight
        png(file, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=cm.colors(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=cm.colors(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co <- cm.colors(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      if (input$checkGroup==8){
        co <- colorRampPalette(c("black","#3a0303","#640000","#981800","#ca4b00",
                                 "#fc7f01","#ffb234","#ffe566","#ffff98","#ffffcb","white"))
        iwidth  <- imagewidth
        iheight <- imageheight
        png(file, width=iwidth, height=iheight)
        par(cex = textsize)
        image.plot(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],main=input$text1,
                   xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
                   zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        
        image(lon,lat,array(1:2,dim(data[,,1])),xlab=" ",ylab=" ",col=na.color,axes=F,
              xlim=input$slider1,ylim=input$slider2,add=T)
        
        image(lon,lat,data[,,which(date.time==input$timestep,arr.ind = T)],
              xlab=" ",ylab=" ",xlim=input$slider1,ylim=input$slider2,
              zlim=c(input$num_rmim,input$num_rmax),col=co(input$num_brk),axes=F,add=T)
        
        if(!as.logical(input$int)){map("world",add=T,interior=F,resolution=0,col=bordercolor,lwd=linesize)}
        if(as.logical(input$int)){plot(world,add=TRUE,lwd=linesize,col=bordercolor)}
        
        if(plot_grid){grid (NULL,NULL, lty = 3, col = grid_col)}
        axis(1,mgp=c(0,-2.5,0),tck=c(0.01),col.axis=bordercolor,cex.axis=0.8*textsize,
             at=xtick,labels=xlab)
        axis(2,mgp=c(0,-2.5,0),tck=c(0.01),las=1,col.axis=bordercolor,cex.axis=0.8*textsize,
             at=ytick,labels=ylab)
        box(col=bordercolor,lwd=linesize)
        
        if(input$location==TRUE){
          if(length(lon_loc_vec())>0){
            for (i in 1:length(lon_loc_vec())){
              points(lon_loc_vec()[i],lat_loc_vec()[i],pch=16,col=bordercolor)
              text(lon_loc_vec()[i],lat_loc_vec()[i],name_loc_vec()[i], pos=1,
                   col=bordercolor,cex=textsize)
            }
          }
        }
        
        mtext(input$text2)
        
        # plot R-Instat
        if(plot_rinstat()==1){  
          co_ri <- co(input$num_brk)
          vec <- seq(input$num_rmim,input$num_rmax,length.out=input$num_brk+1)
          data_station <- co.data()$data_station
          lon_station  <- co.data()$lon_station
          lat_station  <- co.data()$lat_station
          data_station[data_station>=input$num_rmax] <- input$num_rmax
          data_station[data_station<=input$num_rmim] <- input$num_rmim                 
          for(i in 1:length(data_station)){
            point_col <- co_ri[findInterval(data_station[i],vec,all.inside = T)]
            points(lon_station[i],lat_station[i],pch=21,bg=point_col,
                   col="gray30",cex=3,lwd=2)
          }
        }
        
        on.exit(dev.off())
      }
      
      # If orthographic projection
      if (input$proj == "ortho"){
        # prepare plot
        ori  <- c(input$xort,input$yort,input$rort)             #orientation
        nx <- length(lon)
        ny <- length(lat)
        landcol  <- "navajowhite3" 
        oceancol <- "cadetblue3"
        outcol   <- "cornsilk4"
        
        rep.row<-function(x,n){
          matrix(rep(x,each=n),nrow=n)
        }
        
        lonv  <- replicate(length(lat),lon)
        latv  <- rep.row(lat,length(lon))
        datav <- as.vector(data[,,which(date.time==input$timestep,arr.ind = T)])
        
        a <- mapproject(x=lonv,y=latv,projection="orthographic",orientation=ori)
        m <- map("world",plot=F)
        
        # filter Nas
        if (sum(is.na(a$x))>0|sum(is.na(a$y))>0){
          dummy <- NULL
          dummy <- !is.na(a$x)
          a$x   <- a$x[dummy]
          a$y   <- a$y[dummy]
          datav <- datav[dummy]
          dummy <- NULL
          dummy <- !is.na(a$y)
          a$x   <- a$x[dummy]
          a$y   <- a$y[dummy]
          datav <- datav[dummy]
        }
        
        # define grid factors
        xr <- abs(range(lon,na.rm=T)[1])+abs(range(lon,na.rm=T)[2])
        yr <- abs(range(lat,na.rm=T)[1])+abs(range(lat,na.rm=T)[2])
        l1 <- 3.1  # max value for nx/xf
        l2 <- 2.0  # max value for ny/yf
        
        x1 <- c(40,360)
        y1 <- c(1,l1)
        c1 <- lm(y1~x1)$coeff[[1]]
        c2 <- lm(y1~x1)$coeff[[2]]
        
        if (xr>40 & xr<=360){
          xf <- c2*xr+c1
          xf <- round(xf,digits=1)
        } else { xf <- 1}
        
        x1 <- c(40,180)
        y1 <- c(1,l2)
        c1 <- lm(y1~x1)$coeff[[1]]
        c2 <- lm(y1~x1)$coeff[[2]]
        
        if (yr>40 & yr<=180){
          yf <- c2*yr+c1
          yf <- round(yf,digits=1)
        } else { yf <- 1}
        
        iwidth  <- 800
        iheight <- 800
        
        # define color palette
        if (input$checkGroup==1){pcol <- tim.colors(input$num_brk)}
        if (input$checkGroup==2){pcol <- heat.colors(input$num_brk)}
        if (input$checkGroup==3){pcol <- terrain.colors(input$num_brk)}
        if (input$checkGroup==4){
          co <- colorRampPalette(c("darkblue","white","darkred"))
          pcol <- co(input$num_brk)}
        if (input$checkGroup==5){
          co <- colorRampPalette(c("darkred","white","darkblue"))
          pcol <- co(input$num_brk)}
        if (input$checkGroup==6){pcol <- rainbow(input$num_brk)}
        if (input$checkGroup==7){pcol <- cm.colors(input$num_brk)}
        if (input$checkGroup==8){
          co <- colorRampPalette(c("black","#3a0303","#640000","#981800",
                                   "#ca4b00","#fc7f01","#ffb234","#ffe566",
                                   "#ffff98","#ffffcb","white"))
          pcol <- co(input$num_brk)}
        
        tlab <- break_num(input$num_tick,input$num_tick,input$num_rmim,input$num_rmax)
        
        # Plot orthographic image
        png(file, width=iwidth, height=iheight)
        
        quilt.plot(a$x,a$y,datav,xlim=c(-1,1),ylim=c(-1,1),nx=nx/xf,ny=ny/yf,xlab=" ",ylab=" ",
                   main=input$text1,col=pcol,
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F)
        image(lon,lat,array(1:2,dim(data[,,which(date.time==input$timestep,arr.ind = T)]))
              ,xlab=" ",ylab=" ",col="black",axes=F,xlim=c(-1,1),ylim=c(-1,1),add=T)
        polygon(sin(seq(0,2*pi,length.out=100)),cos(seq(0,2*pi,length.out=100)), 
                col=oceancol, border=rgb(1,1,1,0.5), lwd=1)
        suppressWarnings(map("world",projection="orthographic",orientation=ori,add=T,interior=F
                             , fill=T, col=landcol,lwd=linesize,resolution=0,border=NA))
        quilt.plot(a$x,a$y,datav,xlim=c(-1,1),ylim=c(-1,1),nx=nx/xf,ny=ny/yf,xlab=" ",ylab=" ",
                   main=input$text1,col=pcol,
                   axis.args=list(cex.axis=1,at=as.numeric(tlab[tlab!=""]),labels=tlab[tlab!=""], 
                                  mgp=c(1,0.4,0), tck=c(-0.3)),
                   legend.lab=paste(varname," [",unit,"]",sep=""),legend.line=-2,axes=F,add=T)
        if(!as.logical(input$int)){suppressWarnings(map("world",projection="orthographic",orientation=ori,add=T,interior=F
                                                        ,col=outcol,lwd=linesize,resolution=0))
        } else {
          suppressWarnings(map("world",projection="orthographic",orientation=ori,add=T,interior=T
                               ,col=bordercolor,lwd=linesize,resolution=0))
        }
        if(plot_grid){map.grid(m,nx=18,ny=9,lty=3,col=grid_col,cex=linesize)}
        mtext(input$text2)
        on.exit(dev.off())
      }
    },
    contentType = "image/png")    
})
onStop(function() {
  cat("Thanks for using the CM SAF R Toolbox.\n")
  graphics.off()
})

