# (Steffen Kothe / CM SAF 2018-07-26)

shinyUI(fluidPage(
  theme = shinythemes::shinytheme("flatly"),

  sidebarLayout(
    sidebarPanel(img(src="R_Toolbox_Logo.png", width="100%"), 
      h3("Visualizer Options:"),
      selectInput('timestep', label = h5('Select Time Step'), choices = date.time,
        selectize = FALSE),
      sliderInput("slider1", label = h5("Longitude"),
                  min = round(min_lon_slider), max = round(max_lon_slider), value = c(min_lon,max_lon)),
      sliderInput("slider2", label = h5("Latitude"),
                  min = round(min_lat_slider), max = round(max_lat_slider), value = c(min_lat,max_lat)),
      div(style="display: inline-block;vertical-align:top; width: 160px;",
          numericInput("num_rmim", label = h5("Scale Range Min"), value = round(min_data,digits=1), 
                   step = 0.5, width = "60%")),
      div(style="display: inline-block;vertical-align:top; width: 160px;",
          numericInput("num_rmax", label = h5("Scale Range Max"), value = round(max_data,digits=1), 
                   step = 0.5, width = "60%")),
      div(style="display: inline-block;vertical-align:top; width: 160px;",
          numericInput("num_brk", label = h5("Number of Colors"), value = 32, min = 2, max = 64, step = 1, width = "60%")),
      div(style="display: inline-block;vertical-align:top; width: 160px;",
          numericInput("num_tick", label = h5("Number of Ticks"), value = 5, min = 2, max = 64, step = 1, width = "60%")),
      checkboxInput("int","Plot Country Borders"),
      checkboxInput("instat","Plot R-Instat"),
      conditionalPanel(condition = "input.instat == true",
                       shinyFilesButton(id="file2",label="Choose an R-Instat file...",
                                        multiple=F,title="Please select an .RData file.")),
      checkboxInput("location","Plot Own Location"),
      conditionalPanel(condition = "input.location == true",
                       div(style="display: inline-block;vertical-align:top; width: 160px;",
                           numericInput("lon_loc", label = h5("Lon"), 
                                        value = 8.75, min = -180, max = 180, width = "60%")),
                       div(style="display: inline-block;vertical-align:top; width: 160px;",
                           numericInput("lat_loc", label = h5("Lat"), 
                                        value = 50.11, min = -90, max = 90, width = "60%")),
                       div(style="display: inline-block;vertical-align:top; width: 200px;",
                           textInput("name_loc",label=h5("Name"),value="Offenbach",width="80%")),
                       div(style="display: inline-block;vertical-align:center;margin-top: 40px; width: 160px;",
                           actionButton("add_loc", label = "Add now"))),
      selectInput("checkGroup", label = h5("Colorbar"), 
                  choices = list("Tim" = 1, "Heat" = 2, "Terrain" = 3, "Difference" = 4, "Difference inverted" = 5, "Rainbow" = 6, 
                                 "CM" = 7, "Sunny" = 8), selected = 1),
      selectInput("proj",label = h5("Projection"),
                  c(Rectangular = "rect", Orthographic = "ortho")),
      conditionalPanel(condition = "input.proj == 'ortho'",
                       div(style="display: inline-block;vertical-align:top; width: 160px;",
                           numericInput("yort", label = h5("Center Lon"), value = 0, 
                                        step = 5, min= -175, max= 180, width = "50%")),
                       div(style="display: inline-block;vertical-align:top; width: 160px;",
                           numericInput("xort", label = h5("Center Lat"), value = 0, 
                                        step = 5, min= -85, max= 85, width = "50%")),
                       div(style="display: inline-block;vertical-align:top; width: 160px;",
                           numericInput("rort", label = h5("Rotation"), value = 0, 
                                        step = 5, min= -85, max= 85, width = "50%"))),
      textInput("text1", label = h5("Title"), value = varname),
      textInput("text2", label = h5("Subtitle"), value = " "),
      textInput("text3", label = h5("Scale Caption"), value = paste(varname," [",unit,"]",sep="")),
      downloadButton('downloadPlot', 'Save as png-file'),
      actionButton("exit", label = "EXIT", class = "btn btn-danger")),
    mainPanel(
      tabsetPanel(
        tabPanel("Plot",
                 h4("\n"),
                 imageOutput("myImage")),
        tabPanel("Statistics",
                 fluidRow(column(6,
                                 h4("Some numbers for the selected region."),
                                 verbatimTextOutput("statistics")),
                          column(10,
                                 imageOutput("myHist")),
                          column(10,
                                 imageOutput("myComp")))),
        tabPanel("File Summary",
                 fluidRow(column(12,        
                                 h4("Short File Info"),
                                 verbatimTextOutput("summary1")),
                          column(12,
                                 h4("Detailed File Info"),
                                 verbatimTextOutput("summary2")))),
        tabPanel("About",
                 fluidRow(column(8,
                                 h4("The CM SAF Visualizer"),
                                 verbatimTextOutput("about")),
                          column(8,
                                 h4("Good to know"),
                                 verbatimTextOutput("tipps")),
                          column(6,
                                 uiOutput("link"))))
      )
    )
  )))

