# This shiny application is an interface to the three main R-scripts of
# the CM SAF R Toolbox. It will be started by the R-script CMSAF-R-TOOLBOX.R
#
# You should not use this R-script on its own!
#
# Have fun with the CM SAF R TOOLBOX!   
#                                              (Steffen Kothe / CM SAF 2018-08-08)
#__________________________________________________________________________________








ui <- fluidPage(
  theme = shinythemes::shinytheme("flatly"),
  sidebarLayout(
    sidebarPanel(width = 4,
                 img(src="R_Toolbox_Logo.png", width="100%"),
                 br(),
                 br(),
                 actionButton("action1", label = "Prepare", class = "btn btn-primary btn-lg btn-block",
                                  style = "font-size: 30px"),
                 br(),
                 actionButton("action2", label = "Analyze", class = "btn btn-primary btn-lg btn-block",
                              style = "font-size: 30px"),
                 br(),
                 actionButton("action3", label = "Visualize", class = "btn btn-primary btn-lg btn-block",
                              style = "font-size: 30px"),
                 br(),
                 actionButton("go", label = "GO", class = "btn btn-success btn-lg btn-block",
                              style = "font-size: 30px"),
                 br(),
                 actionButton("exit", label = "EXIT", class = "btn btn-danger btn-lg btn-block",
                                  style = "font-size: 30px"),
                 br(),
                 shinyFilesButton(id="file1",label="Browse files...",
                                  multiple=F,title="Please select an approriate file.")),
    mainPanel(width=8,verbatimTextOutput("text1"),
              htmlOutput("text2"),
              br(),
              verbatimTextOutput("txt_file"))
  ))

server <- function(input, output, session) {
  
  napp <- reactiveVal(0)
  file_selected <- reactiveVal()
  
  # depending on OS
  if(Sys.info()['sysname']=="Windows"){
    volumes = c('home' = '~','C:' = 'C:/','Users' = 'C:/Users/')
  } else {
    volumes = getVolumes()
  }
  
  observe({  
    shinyFileChoose(input, "file1", roots = volumes, session = session)
    
    #if(!is.null(input$file1) && as.character(input$file1) != "0"){
     if(!is.integer(input$file1)){
      fs <- parseFilePaths(volumes, input$file1)
      file_selected(parseFilePaths(volumes, input$file1))
      output$txt_file <- renderText(paste("Selected file: ",as.character(fs$datapath),sep=""))
    } else {
      if(!is.null(last_file)){
        file_selected(last_file)
        output$txt_file <- renderText(paste("Selected file: ",file_selected(),sep=""))
      } else {
        file_selected("NoFile")
        output$txt_file <- renderText("No file selected.")
      }
    }
  })

  output$text1 <- renderPrint({
    cat("The intention of the CM SAF R Toolbox is to help you using","\n")
    cat("CM SAF NetCDF formatted data.","\n")
    cat("\n")
    cat("This includes: 1. Preparation of ordered CM SAF data files","\n")
    cat("               2. Analysis of prepared CM SAF data","\n")
    cat("               3. Visualization of the results","\n")
    cat("\n")
    cat("Select one of the three steps and launch it by clicking the GO button.","\n")
    cat("Stop the application by clicking the EXIT button.","\n")
    cat("\n")
    cat("Suggestions for improvements and praise for the developers","\n")
    cat("can be send to contact.cmsaf@dwd.de.","\n")
    cat("\n")
    cat("                              - Steffen Kothe - 2018-06-12 -","\n")
    cat("\n")
  })
  
  observeEvent(input$action1, {
    text <- "<h2>Prepare</h2>
      <p>Please select a TAR file <strong>(.tar)</strong> and click on <strong>GO</strong></p>
      <p>to start the R-script Prep.Data.R.</p>
      <p>This is the first step after you downloaded your ordered tar-file(s).<p/>
      <br>
      <p>This application will help you to extract, unzip and merge the data.</p>
      <p>In addition, you can select a time range and region from your data.</p>
      <br>
      <p>The script guides through all steps. If you are not sure about an input,</p>
      <p>just hit <strong>ENTER</strong>. Then a default will be taken.</p>"
    napp(napp() * 0)
    napp(napp() + 1)
    output$text2 <- renderText({ text })
  })
  observeEvent(input$action2, {
    text <- "<h2>Analyze</h2>
      <p>Please select a NetCDF file <strong>(.nc)</strong> and click on <strong>GO</strong></p>
      <p>to start the R-script Apply.Function.R.</p>
      <p>This is the second step after you prepared your data.<p/>
      <p>The input for this application has to be a NetCDF file</p>
      <p>(usually the output of step one (Prepare)).</p>
      <br>
      <p>This application will help you to analyze and manipulate your data.</p>
      <p>The output is usually written in a NetCDF file in the <em>output</em> folder.</p>
      <br>
      <p>The script guides through all steps. If you are not sure about an input,</p>
      <p>just hit <strong>ENTER</strong>. Then a default will be taken.</p>"
    napp(napp() * 0)
    napp(napp() + 2)
    output$text2 <- renderText({ text })
  })
  observeEvent(input$action3, {
    text <- "<h2>Visualize</h2>
      <p>Please select a NetCDF file <strong>(.nc)</strong> and click on <strong>GO</strong></p>
      <p>to start the R-script CMSAF_Visualizer.R.</p>
      <br>
      <p>This application can be used to display NetCDF data.</p>
      <p>In addition, it provides information on the data and the NetCDF file.</p>
      <br>
      <p>To stop the CM SAF Visualizer click on <strong>EXIT</strong>.</p>"
    napp(napp() * 0)
    napp(napp() + 3)
    output$text2 <- renderText({ text})
  })
  observeEvent(input$go, {
    if(napp()==0){
      output$text2 <- renderText({ "<h4>Please select an application or EXIT.</h3>" })
    } else {
        stopApp(returnValue = c(napp(),file_selected()))
      }
  })
  observeEvent(input$exit, {
    stopApp(returnValue = invisible(99))
  })
}

shinyApp(ui, server)